##########################################################
############# Data preparation - ready for ARM ###########
##########################################################

## 0.1) data which has not been in sef-format is additionally converted to sef
## 0.2) for aarau obs time has been estimated
## 0.3) subdaily qc has been added to files, where it has been missing
## -> the codes for these steps are indiv.for every station

## 1) calculation of daymeans
## 2) reduce pressure values to 0°C (is it okay to do this on the daymeans?)
## 3) daily qc and daily spatial consistency checks
## 4) homogenization of historical stations (only take the type 1 sig.breakpoints and add adjustements directly)
## 5) create reference series as a sef
## 6) merge pressure series from bern, zurich, etc.
## 8) homogenize historical stations with respect to reference station based on EKF400
## 8) create TOT file for further pre-processing and analog selection based on daymeans/refseries and daymeans/homref

rm(list=ls())
main <- "/dir/"
setwd(main)

########################################
## 0.1) - 0.3)
########################################
## creates the additional files and stores them in data_sef/qced_subdaily/
## HOH_ta and TOR_p needs to be copied, because it has no flags, ie.not written to new folder.
## cp sef_subdaily/WeaR_HOH_17810101-20201231_ta.tsv qced_subdaily/WeaR_HOH_17810101-20201231_ta_qc.tsv
## cp sef_subdaily/WeaR_TOR_17530101-20091231_p.tsv qced_subdaily/WeaR_TOR_17530101-20091231_p_qc.tsv

stations <- c("aarau", "basel","bologna","genf",
              "hohenpeissenberg","karlsruhe","milano",
              "padua","torino","zurich","zurich_nzg","fribourg","gurzelen","gsb")

#source("correct_schaffhausen.R") this was only once
lapply(paste0(main,"code/data_prep/scripts/",stations,".R"), source)

########################################
## 1) calculation of daymeans
########################################
## use ta for Torino, not Tx,Tn
## copy HOH_ta to qced_subdaily and rename manually
source("code/data_prep/scripts/C001_calc_dailymeans_arm.R")
source("code/data_prep/scripts/C001_calc_dailymeans_rrt.R")

########################################
## 2) reduce pressure values to 0°C 
########################################

source("code/data_prep/scripts/C002_pressure_reduction_20crv2.R")

########################################
## 3) daily qc and daily spatial consistency checks
########################################
## copy first all files in sef to the qc folder.
## daily does standard qc + add manual qc as well. 
source("code/data_prep/scripts/C003_daily_qc.R")
## spatial qc and it also checks whether all days in sef files are real, ie. not 1841-06-31
source("code/data_prep/scripts/C003_spatial_consistency_p.R") ## f=4 
source("code/data_prep/scripts/C003_spatial_consistency_ta.R") ## f=4
## no spatial qc for precipitation, because of low spatial coherence and few data
source("code/data_prep/scripts/C003_rr0_qc.R")

#######################################################
## 4) homogenize the station data "independently"
#######################################################
## create hom input for references EKF400 and for SEF files
## max 25% missing data to calculate a monthly value
source("code/data_prep/scripts/C004_create_hom_input.R")
## this includes calculation of bps and adding adjustments.
## output is stored in the folder daymeans/hom
## there is an individual plot made at the end of the script for each series.
source("code/data_prep/scripts/C004_homogenization.R")

#############################################################################################################
## 5) create reference series
#############################################################################################################
## based on MCH, Tabs-MCH, ECAD and EOBS-Data, reference series are created for all the historical series
## the new series are all stored in sef, in meta it is indicated, from where the data stems, i.e. eobs, etc.
## note, that the sef only cover the ref-period, 1961-2020. Thus, the period in between is missing.
## pressure is converted to station altitude using the temperature ref-series, where available.
source("code/data_prep/scripts/C005_create_referenceseries_ta.R")
source("code/data_prep/scripts/C005_lugano_hom_input.R")
source("code/data_prep/scripts/C005_homogenization_ref_ta.R") 
source("code/data_prep/scripts/C005_karlsruhe_ta_sef.R") ## this is needed for creation of the p file for the gap filling.

source("code/data_prep/scripts/C005_create_referenceseries_p.R") 
source("code/data_prep/scripts/C005_create_referenceseries_rr0.R")
source("code/data_prep/scripts/C005_create_referenceseries_rr.R")

#############################################################################################################
## 6) merge these homogenized stations
#############################################################################################################
## merge pressure for zurich,basel,bern,genf
source("code/data_prep/scripts/C006_merge_basel_p.R")
source("code/data_prep/scripts/C006_merge_zurich_p.R")
source("code/data_prep/scripts/C006_merge_bern_p.R")
source("code/data_prep/scripts/C006_merge_genf_p.R")

#### create sef from yuris merged files for zurich, bern,basel, geneva. 
source("code/data_prep/scripts/C006_create_sef_from_yuri.R")

## for precip
source("code/data_prep/scripts/C006_merge_rr.R")

#############################################################################################################
## 7) homogenize the station data with EKF400v2 to make them consistent with the reference period
#############################################################################################################
## all stations are homogenized with respect to the reference station in the present day period.
source("code/data_prep/scripts/C007_homogenization_to_reference.R")

#############################################################################################################
## 8) create one RData file for analog selection
#############################################################################################################
source("code/data_prep/scripts/C008_create_TOT_file.R")




