### help fun genereal

require(lubridate)

leapyear <- function(x){
  return(ifelse((x%%4==0 & x%%100!=0) | x%%400==0,TRUE,FALSE))
}

pad2 <- function(x, width=2){
  stringr::str_pad(x,width,pad="0")
}


cllps <- function(x,pat=""){
  if(is.matrix(x)){
    sapply(1:nrow(x), function(z) paste0(x[z,],collapse = pat))
  }else{
    paste0(x,collapse = pat)
  }
}

t_col <- function(color, percent = 50, name = NULL) {
  rgb.val <- col2rgb(color)
  t.col <- rgb(rgb.val[1], rgb.val[2], rgb.val[3],
               max = 255,
               alpha = (100 - percent) * 255 / 100,
               names = name)
  invisible(t.col)
}


pdate <- function(dat){as.Date(paste0(dat[,1],"-",pad2(dat[,2]),"-",pad2(dat[,3])))}

load_object <- function(file) {
  tmp <- new.env()
  load(file = file, envir = tmp)
  tmp[[ls(tmp)[1]]]
}


mean_Xpercna <- function(x, na=0.25){
  if(sum(is.na(x))/length(x) <= na){
    return(mean(x, na.rm=T))
  } else{return(NA)}}

sum_Xpercna <- function(x, na=0.25){
  if(sum(is.na(x))/length(x) <= na){
    return(sum(x, na.rm=T))
  } else{return(NA)}}



yday_without_29feb <- function(predperiod){
  leapyears <- leapyear(lubridate::year(predperiod))  
  jind <- which(leapyears & substr(predperiod,6,10)>"02-28")
  jdays <- lubridate::yday(predperiod)
  if (length(jind) > 0) jdays[jind] <- jdays[jind]-1 
  return(jdays)
}



quant_window_fun <- function(dat,dates,doy){
  window <-8
  doysel <- (doy-window):(doy+window)
  doysel[which(doysel<1)] <- doysel[which(doysel<1)]+365
  doysel[which(doysel>365)] <- doysel[which(doysel>365)]-365
  doyind <- which(yday_without_29feb(dates) %in% doysel)
  q <- quantile(dat[doyind],probs=c(0.1,0.25,0.5,0.75,0.9), na.rm = T)
  return(q)
}



check_dates <- function(out){
  
  rdates <- pdate(out[,1:3])[c(1,nrow(out))]
  hdates <- seq(rdates[1],rdates[2],by="day")
  
  ## check for realistic dates first
  find <- which(!pdate(out[,1:3]) %in% hdates)
  if(length(find)>0) out <- out[-find,]
  
  ## add missing days to make a complete time series
  missdays <- !hdates %in% pdate(out[,1:3])
  
  if(sum(missdays)>0){
    newout <- data.frame(Year=year(hdates),
                         Month=month(hdates),
                         Day=day(hdates),
                         Hour=rep(NA, length(hdates)),
                         Minute=rep(NA, length(hdates)),
                         Value=rep(NA,length(hdates)),
                         Meta= rep("NA",length(hdates)))
    
    newout[!missdays,"Value"] <- out$Value
    newout$Meta <- as.character(newout$Meta)
    newout[!missdays,"Meta"] <- as.character(out$Meta)
    out <- newout
  }
  out$Meta <- as.character(out$Meta)
  
  return(out)  
}

rm_seas <- function(tempdata,dates) {
  
  if(!all(is.na(tempdata))){
    
    doys <- yday(dates)
    ndoy <- diff(seq(dates[1], as.Date(paste((year(dates[length(dates)])+1),"-01-01",sep="")), by = "year"))
    ndoy <- rep(ndoy,ndoy)
    ndoy <- ndoy[1:length(dates)]
    xdoys <-doys/ndoy
    
    #empdatacor <- tempdata
    
    pcos <- cos(2*pi*xdoys)
    psin <- sin(2*pi*xdoys)
    pcos2 <- cos(4*pi*xdoys)
    psin2 <- sin(4*pi*xdoys)
    
    temp.jg.lm <- lm(tempdata ~ pcos + psin + pcos2 + psin2)
    temp.jg.cor <- as.numeric(tempdata[as.numeric(names(temp.jg.lm$fitted.values))] - temp.jg.lm$fitted.values)
    tempdata[!is.na(tempdata)] <- temp.jg.cor
  }
  
  return(tempdata)
}   



get_ydayclim <- function(tempdata,dates) {
  
  if(!all(is.na(tempdata))){
    
    doys <- yday(dates)
    ndoy <- diff(seq(dates[1], as.Date(paste((year(dates[length(dates)])+1),"-01-01",sep="")), by = "year"))
    ndoy <- rep(ndoy,ndoy)
    ndoy <- ndoy[1:length(dates)]
    xdoys <-doys/ndoy
    
    pcos <- cos(2*pi*xdoys)
    psin <- sin(2*pi*xdoys)
    pcos2 <- cos(4*pi*xdoys)
    psin2 <- sin(4*pi*xdoys)
    
    temp.jg.lm <- lm(tempdata ~ pcos + psin + pcos2 + psin2)
    if(any(doys==366)){
      leapy <- which(doys==366)[1]
    } else {
      leapy <- 366
    }
    clim <- temp.jg.lm$fitted.values[(leapy-366+1):leapy]
  } else {
    clim <- rep(NA, 366)
    
  }
  return(clim)
  
}   
