#######################################################################
##### create sef files from rrt and convert them to rrt
########################################################################

require(XLConnect)
require(lubridate)
require(dataresqc)

rm(list=ls())

source("code/data_prep/scripts/C000_helpfun.R")
source("code/data_prep/scripts/C005_helpfun_ref.R")

finpath <- "code/data_prep/daymeans/sef/"

## function
rrt_day <- function(x){any(x!="")+0}
rrt_day2 <- function(x){any(!is.na(x))+0}
## CHIMES.RData and seffiles are formated differently- thus the two functions

####################################
### CHIMES 
####################################
load("data/CHIMES_v1.1/CHIMES.RData")
inventory <- readWorksheetFromFile("code/data_input/arm_stations_inventory_withnbcn.xlsx",sheet=1, header=T,colTypes="character",startRow=2)

## select variables + stns
tt <- which(!is.na(chimes[,"precipitation_type"]))
tt_stn <-  unique(chimes[tt,"station"])
tt_stn <- tt_stn[!grepl("Aarau", tt_stn)]

for(ss in 1:length(tt_stn)){
  
  selind <- which(chimes$station==tt_stn[ss] & !is.na(chimes$precipitation_type))
  seldat <- chimes[selind,]
  print(unique(seldat$precipitation_type))
  stnnam <- as.character(seldat$station[1])
  ## i think Fribourg is wrong, ie. this does not exist
  
  sefdata <- data.frame(year=seldat$year,
                          mo=seldat$month,
                          d=seldat$day,
                          h=seldat$hour,
                          m=seldat$minutes,
                          Value=seldat$precipitation_type,
                          Meta=NA)
  sefdata$Value <- as.character(sefdata$Value)
  norain <- which(sefdata$Value %in% c("0","kein Regen", "Thau","Th.","Thau od Reif","Rf"))
  sefdata$Value[norain] <- ""
    
  aggd <- aggregate(sefdata$Value, list(sefdata$year,sefdata$mo,sefdata$d), rrt_day)
  aggd <- aggd[order(aggd$Group.1,aggd$Group.2,aggd$Group.3),]
  metahelp <- aggregate(seldat$precipitation_type, list(sefdata$year,sefdata$mo,sefdata$d), cllps, pat="-")
  metahelp <- metahelp[order(metahelp$Group.1,metahelp$Group.2,metahelp$Group.3),"x"]
  sefdata_new <- cbind(aggd[,1:3],rep(NA, nrow(aggd)),rep(NA, nrow(aggd)),aggd$x,paste0("orig.val=",metahelp))
  colnames(sefdata_new) <- c("Year","Month","Day","Hour","Minute","Value","Meta")
  sefdata_new <- check_dates(sefdata_new)  
  
  write_sef(Data = sefdata_new[, 1:7],
            outpath = finpath,
            variable = "rr0",
            cod = stnnam,
            nam = seldat$place[1],
            lat = seldat$latitude[1],
            lon = seldat$longitude[1],
            alt = seldat$altitude[1],
            link = "",
            sou = "CHIMES",
            units = "coded",
            stat = "sum",
            metaHead = "",
            meta=sefdata_new$Meta,
            period = "day",
            time_offset =0,
            keep_na = TRUE,
            note="daily")
  

}

####################################
### Aarau
####################################
stn <- unique(chimes$station[!is.na(chimes$rain_occurrence)])
selind <- which(chimes$station==stn[1])
seldat <- chimes[selind,]

## all data that are not na
seldat2 <- seldat[,which(apply(seldat,2,function(x){!all(is.na(x))}))]
## read in newly digitized data
tab <- XLConnect:: readWorksheetFromFile("data/data_sef/aarau/AG01_Aarau_1813-1816_Noemi.xlsx", sheet=1, startRow=9, header=T, colTypes="character")
## reorder
tnew <- as.vector(unlist(t(tab[,c("Time1","Time2","Time3")])))
vnew <- as.vector(unlist(t(tab[,c("Col7","Col8","Col9")])))
tabnew <- cbind(data.frame(rep(tab$Year, each=3),rep(tab$Month, each=3),rep(tab$Day, each=3)), tnew, vnew)
tabnew$vnew2 <- as.integer(as.character(tabnew$vnew))

print(unique(seldat$rain_occurrence))
print(unique(seldat$snow_occurrence))
stnnam <- as.character(seldat$station[1])
## combine snow and rain
newvals <- seldat$rain_occurrence +0
newvals[which(seldat$snow_occurrence)] <- 1
newvals[which(seldat$precipitation_occurrence)] <- 1
newvals[which(seldat$precipitation_occurrence==F)] <- 0
newvals[which(seldat$precipitation_type%in%c("RS","S","SR","R"))] <- 1
newvals[which(seldat$precipitation_type%in%c(""))] <- 0
newvals[which(seldat$precipitation_type%in%c("E"))] <- NA
newvals[which(pdate(seldat[,c("year","month","day")]) %in% pdate(tabnew[,1:3]))] <- tabnew$vnew2

sefdata <- data.frame(year=seldat$year,
                      mo=seldat$month,
                      d=seldat$day,
                      h=seldat$hour,
                      m=seldat$minutes,
                      Value=newvals,
                      Meta=paste0("snow.orig=",seldat$snow_occurrence_orig,"|rain.orig=",seldat$rain_occurrence_orig,
                                  "|precip.orig=",seldat$precipitation_occurrence_orig,"|precip.type=",seldat$precipitation_type)
                      )

aggd <- aggregate(sefdata$Value, list(sefdata$year,sefdata$mo,sefdata$d), function(x){
  x <- x[!is.na(x)]
  if(length(x)<1){
    return(NA)
  } else{
    return(any(x==1)+0)
  }
  })

aggd <- aggd[order(aggd$Group.1,aggd$Group.2,aggd$Group.3),]
sefdata_new <- cbind(aggd[,1:3],rep(NA, nrow(aggd)),rep(NA, nrow(aggd)),aggd$x,rep(NA, nrow(aggd)))
colnames(sefdata_new) <- c("Year","Month","Day","Hour","Minute","Value","Meta")
sefdata_new <- check_dates(sefdata_new)  

write_sef(Data = sefdata_new[, 1:7],
          outpath = finpath,
          variable = "rr0",
          cod = stnnam,
          nam = seldat$place[1],
          lat = seldat$latitude[1],
          lon = seldat$longitude[1],
          alt = seldat$altitude[1],
          link = "",
          sou = "CHIMES",
          units = "coded",
          stat = "sum",
          metaHead = "",
          meta=sefdata_new$Meta,
          period = "day",
          time_offset =0,
          keep_na = TRUE,
          note="daily")


####################################
### Basel
####################################
files <- list.files("data/data_sef/basel/", pattern="rrt", full.names = T)
sef <- read_sef(files, all=T)
meta <- read_meta(files)

sefagg <- aggregate(sef$Value, list(sef$Year,sef$Month,sef$Day), rrt_day) 
sef <- sefagg[order(sefagg$Group.1, sefagg$Group.2),]
sefdata <- data.frame(year=sef$Group.1,
                      mo=sef$Group.2,
                      d=sef$Group.3,
                      h=rep(NA, nrow(sef)),
                      m=rep(NA, nrow(sef)),
                      Value=sef$x,
                      Meta=sef$x)
## set NA to 0, but this is just temporary
sefdata <- check_dates(sefdata)  
sefdata[is.na(sefdata$Value),"Value"] <- 0

## read in digitzed values from ECHDB
ann_ech <- readWorksheetFromFile(list.files("data/data_sef/basel/", pattern="DBout", full.names = T),sheet=1, header=T,colTypes="character",startRow=1)
ann_rrt <- data.frame(date=ann_ech$DATE.BEGIN, value=rep(0,nrow(ann_ech)), orig=ann_ech$CAT_ID, control=rep(0,nrow(ann_ech)))
rr_ids <- unique(ann_ech$CAT_ID)
rr_ind <- grep("Regen|Schnee|Niederschlag|Hagel",ann_rrt$orig)
ann_rrt[rr_ind,"value"] <- 1
dateind <- which(as.Date(ann_rrt$date) %in% pdate(sefdata[,1:3]))
dateind2 <- which(pdate(sefdata[,1:3]) %in% as.Date(ann_rrt$date))
ann_rrt[dateind,"control"] <- sefdata$Value[dateind2]

unequal <- which((ann_rrt$value-ann_rrt$control)!=0)

## the table above has been adjusted manually by controlling the source files. this is now in the table
control <- read.table(file="data/data_sef/basel/comparison_echdb_giub.txt", sep="\t", header=T)
sefdata[which(pdate(sefdata[,1:3]) %in% pdate(control[,1:3])),"Value"] <- control$check

write_sef(Data = sefdata[, 1:7],
          outpath = finpath,
          variable = "rr0",
          cod = meta["id"],
          nam = meta["name"],
          lat = meta["lat"],
          lon = meta["lon"],
          alt = meta["alt"],
          link = "",
          sou = meta["source"],
          units = "coded",
          stat = meta["stat"],
          metaHead = "",
          meta=sefdata$Meta,
          period = "day",
          time_offset =0,
          keep_na = TRUE,
          note="daily")



####################################
### Zurich
####################################
files <- list.files("data/data_sef/zurich/", pattern="rrt", full.names = T)
seffiles <- lapply(files,read_sef, all=T)
metafiles <- lapply(files, read_meta)
id <- sapply(metafiles, function(x){x["id"]})

for(ff in 1:length(seffiles)){
  
  print(id[ff])
  sef <- seffiles[[ff]]
  meta <- metafiles[[ff]]
  print(unique(sef$Value))
  sefagg <- aggregate(sef$Value, list(sef$Year,sef$Month,sef$Day), rrt_day2) 
  sef <- sefagg[order(sefagg$Group.1, sefagg$Group.2),]
  sefdata <- data.frame(year=sef$Group.1,
                        mo=sef$Group.2,
                        d=sef$Group.3,
                        h=rep(NA, nrow(sef)),
                        m=rep(NA, nrow(sef)),
                        Value=sef$x,
                        Meta=NA)
  #print(head(sefdata,20))
  sefdata <- check_dates(sefdata)  
  
  write_sef(Data = sefdata[, 1:7],
            outpath = finpath,
            variable = "rr0",
            cod = meta["id"],
            nam = meta["name"],
            lat = meta["lat"],
            lon = meta["lon"],
            alt = meta["alt"],
            link = "",
            sou = meta["source"],
            units = "coded",
            stat = "sum",
            metaHead = "",
            meta=sefdata$Meta,
            period = "day",
            time_offset =0,
            keep_na = TRUE,
            note="daily")
  
}


#################################################################
############# also create from rr files rr0 files
#################################################################
allrr <- F
if(allrr){
  
  files <- list.files(finpath, "_rr_", full.names = T)
  files <- files[!grepl("Simmler|Aarau|Fribourg|Ulrich", files)]
  sef <- lapply(files, read_sef, all=T)
  meta <- lapply(files, read_meta)
  
  for(ss in 1:length(sef)){
    
    sf <- sef[[ss]]
    mm <- meta[[ss]]
    sf$conv <- ifelse(sf$Value>0,1,0)  
    
    sefdata <- data.frame(year=sf$Year,
                          mo=sf$Month,
                          d=sf$Day,
                          h=sf$Hour,
                          m=sf$Minute,
                          Value=sf$conv,
                          Meta=paste0("orig.val=",sf$Value,"mm"))
    
    write_sef(Data = sefdata[, 1:7],
              outpath = finpath,
              variable = "rr0",
              cod = mm["id"],
              nam = mm["name"],
              lat = mm["lat"],
              lon = mm["lon"],
              alt = mm["alt"],
              link = mm["link"],
              sou = mm["source"],
              units = "coded",
              stat = "sum",
              metaHead = "converted from rr as a test",
              meta=sefdata$Meta,
              period = "day",
              time_offset =0,
              keep_na = TRUE,
              note="daily")
    
    
  }
}
