##### pressure reduction of none-reduced stations ###############
##### it is done on the daymeans, not hourly data
# Lombach
# Studer
# Tavel
# Delemont
# Schaffhausen
# Rovereto
# Basel Annone
# Hirzel

rm(list=ls())

require(dataresqc)
require(ncdf4)
source("code/data_prep/scripts/C000_helpfun.R")

rea_t2m <- "t2m_ydaymean_1850_1900.nc"

## read 20crv2 (one grid point?) based on daymeans
ncp <- nc_open(paste0("data/20crv3/",rea_t2m))
lons <- ncvar_get(ncp, "longitude")
lons[lons>180] <- lons[lons>180] - 360
lats <- ncvar_get(ncp, "latitude")
coords <- data.frame(lon=rep(lons,each=length(lats)), lat=rep(lats,times=length(lons)))

## read sef files 
sef <- "code/data_prep/daymeans/sef/"
outpath <- "code/data_prep/daymeans/sef/"
seffiles <- list.files(sef)
seffiles_p <- seffiles[grepl("_p_daily.tsv",seffiles)]

## select files with no pressure reduction
ptcinfo <- sapply(seffiles_p, function(x){
  y <- read_meta(paste0(sef,x))["meta"]
  if(grepl("PTC=",y)){
    return(substr(gsub(".*PTC=","",y),1,1))
  } else {return(NA)}
})

seffiles_p <- seffiles_p[ptcinfo=="N"]
seffiles_p <- seffiles_p[!is.na(seffiles_p)]
## reduce pressure with temp seasonal cycle
for (ss in seffiles_p){
  file <- read_sef(paste0(sef,ss), all=T)
  x <- file[which(!is.na(file$Value)), c(2:6,8)]
  ko <- grep("qc", x$Meta)
  if (length(ko) > 0) x <- x[-ko, ] # Remove flagged values
  
  meta <- read_meta(paste0(sef,ss))
  metaHead <- meta["meta"]
  metaHead <- stringr::str_replace(metaHead,"PTC=N","PTC=Y(20crv3 seascycle)")
  xlon <- as.numeric(meta["lon"])
  xlat <- as.numeric(meta["lat"])
  
  ## find closest grid point in 20crv3
  sc <- NA
  k <- 0
  coords_st <- coords
  while(is.na(sc[1]) & k<25) {
    i_grid <- which.min(distGeo(c(xlon,xlat), coords_st))
    i_lon <- which(lons == coords_st$lon[i_grid])
    i_lat <- which(lats == coords_st$lat[i_grid])
    ## Get daily cycle
    sc <- ncvar_get(ncp, "t2m", c(i_lon,i_lat,1), c(1,1,-1)) - 273.15
    coords_st[i_grid,] <- NA
    k <- k + 1
  }
  if (k == 25) { ## if no land is found nearby the station, no correction is applied
    sc <- rep(0, 12*24)
    warning(paste("No land grid points close to", meta["id"]))
  }
  ydays <- lubridate::yday(pdate(x[1:3]))
  sc_ts <- sc[ydays]
  x$Value <- round((1-(0.000182*sc_ts))*x$Value,1)
  write_sef(x, outpath, meta["var"], meta["id"], meta["name"], 
            meta["lat"], meta["lon"], meta["alt"], meta["source"], meta["link"], 
            meta["units"], "mean", period="day", metaHead= metaHead, note="daily", keep_na=TRUE)
  
}



