#### test the data for spatial consistency
## calculate regression between all time series for p and ta 

rm(list=ls())
plot <- F

require("dataresqc")

source("code/data_prep/scripts/C000_helpfun.R")
source("code/data_prep/scripts/C005_helpfun_ref.R")

outpath <- "code/data_prep/daymeans/sef/spatialqc/"

## read files
filepath <- paste0(main,"code/data_prep/daymeans/qced/")
files <- list.files(filepath)
files <- files[grepl("_p_", files) & !grepl("_qcs.", files)]
allsefs <- lapply(files, function(x){
  sef <- read_sef(paste0(filepath,x), all=T)
  return(sef)
})

id <- gsub("_daily.tsv|_qc_daily.tsv","",files)
dates <- stringr::str_sub(gsub("_p_daily.tsv","",files),start =-17)
names(allsefs) <- id

## create matrix of all data
yr <- range(sapply(allsefs, function(x) range(x[,"Year"])))
daterange <- seq(as.Date(paste0(yr[1],"-01-01")),as.Date(paste0("1863-12-31")),by="day")
dattot <- matrix(NA, ncol=length(allsefs),nrow=length(daterange))
colnames(dattot) <- as.character(id)
dindm <- list()

for(ff in 1:length(allsefs)){
  y <- allsefs[[ff]]
  dindm[[ff]] <- daterange %in% pdate(y[,c("Year","Month","Day")])
  dinds<-  pdate(y[,c("Year","Month","Day")]) %in% daterange
  dattot[dindm[[ff]],ff] <- y$Value[dinds]
  }

## correlation between all
if (plot){
  corrmat <- cor(dattot, use="pairwise.complete.obs")
  rownames(corrmat) <- id
  png(paste0(outpath,"corrmatt_allsefs_p.png"), width = 1000,height = 1000)
  cols <- colorRampPalette(RColorBrewer::brewer.pal(n=21-1,"RdBu"))(21-1)
  corrplot::corrplot(corrmat, method = "square", na.label.col="gray55",col=cols)
  dev.off()
}

## linear models between all the datasets (sep into temp + pressure)
outmat <- array(NA, dim=dim(dattot))
estmat <- array(NA, dim=dim(dattot))
flginfo <- array(NA, dim=c(nrow(dattot),length(id),10))

ppstns <- 1:length(id)
f <- 4

mods_p <- list()
pred_p <- list()

##########################
#### for pressure
##########################

for(pp in ppstns){
  
  ppdat <- dattot[,pp]
  ## calc model if at least 5 years of data
  mods_p[[pp]] <- lapply(ppstns[ppstns!=pp],function(z){
    eqx <- sum(which(!is.na(ppdat)) %in% which(!is.na(dattot[,z])))
    if(eqx>(365*5)){m<- lm(ppdat~dattot[,z])} else{m <- NA}
    return(m)
    })
  
  pred_p[[pp]] <- lapply(1:length(mods_p[[pp]]),function(z){
    if(!is.na(mods_p[[pp]][z])){
      m<- predict(mods_p[[pp]][[z]])
    } else{
        m <- NA
        }
    return(m)
  })
  
  predmat <- matrix(NA, ncol=length(pred_p[[pp]]), nrow=nrow(dattot))
  for (pd in 1:ncol(predmat)){
    ind <- as.integer(names(pred_p[[pp]][[pd]]))
    predmat[ind,pd] <- pred_p[[pp]][[pd]]
  }
  
  ## standard error of each regression
  dathelp <- predmat-matrix(rep(ppdat,ncol(predmat)), ncol=ncol(predmat))
  serr <- apply(dathelp,2,function(x){
    nna <- !is.na(x)
    return(sqrt(sum(x^2,na.rm=T)/sum(nna)))
      })
  
  ## select regressions with lowest standard errors, that are able to predict at the specific day of the time series
  ## i.e. loop through time series
  ppdat_nna <- which(!is.na(ppdat))
  for(day in ppdat_nna){
    x <- dattot[day,pp]
    ## stations, that have a prediction for day
    pospred <- which(!is.na(dathelp[day,]))
    sp <- order(serr[pospred])
    len <- ifelse(length(sp)>4,5,length(sp))
    selreg <- pospred[sp][1:len]
    ## remove stations with errors larger than 5
    selreg <- selreg[!serr[selreg]>5]
    ## weighted standard errors of stations with lowest std error
    if(length(selreg)>1){
      sest <- sqrt(length(selreg)/sum(serr[selreg]^-2))
      ## create a weighted estimate of x' 
      ## here, signs are not important, because all is positive, but this is different for temperature
      xest <- sqrt(sum((predmat[day,selreg]^2)/(serr[selreg]^2)))/sqrt(sum(serr[selreg]^-2))
      val <- x >= xest - sest*f && x <= xest + sest*f
      ## write to outmat if it is above,below ci
      outmat[day,pp] <- ifelse(val,0,1)
      estmat[day,pp] <- xest
      flginfo[day,pp,1:length(selreg)] <- id[ppstns[ppstns!=pp]][selreg]
      flginfo[day,pp,6:(6+length(selreg)-1)] <- round(serr[selreg],2)
    }
  }

  }

## plot all time series and add in red, the values that have been flagged
flg_list <- list()
for(pp in ppstns){
  ind <- range(which(!is.na(dattot[,pp])))
  
  if (plot){
    png(paste0(outpath,"/sctest_p/sctest_ts_p_",id[pp],"_flagged_f4__",Sys.Date(),".png"), height = 300, width = 1000,res=150)
    par(mar=c(2,2,1,1))
    plot(daterange[ind[1]:ind[2]],dattot[ind[1]:ind[2],pp], pch=19, cex=0.1, ylab="hPa",xlab="")
    colsel <- ifelse(outmat[ind[1]:ind[2],pp]==1,"red",NA)
    points(daterange[ind[1]:ind[2]],dattot[ind[1]:ind[2],pp], pch=19, cex=0.1, col=colsel)
    yrst <- substr(daterange[ind[1]:ind[2]],6,10)=="01-01"
    abline(v=daterange[ind[1]:ind[2]][yrst], lty=2, col="gray75")
    title(id[pp])
    dev.off()
  }
    flgind <- (ind[1]:ind[2])[outmat[ind[1]:ind[2],pp]==1]
    yrs <- lubridate::year(daterange[flgind])
    mos <- lubridate::month(daterange[flgind])
    days <- lubridate::day(daterange[flgind])
    vals <- dattot[flgind,pp]
    stns <- cllps(flginfo[flgind,pp,1:5],"-")
    serrs <- cllps(flginfo[flgind,pp,6:10],"-")
    xest <- round(estmat[flgind,pp],2)
    sctest_flags <- data.frame(Var=rep("p",length(yrs)),Year=yrs,Month=mos,Day=days,Value=vals,Estimate=xest,Refstns=stns,RSerrs=serrs)
    sctest_flags <- sctest_flags[!is.na(sctest_flags[,2]),]
    flg_list[[pp]] <- sctest_flags
    #write.table(sctest_flags,file=paste0(outpath,"/qc_p_fin/qc_sc_",id[pp],"_",dates[pp],".txt"), quote = F,row.names = F)
    write.table(sctest_flags,file=paste0(outpath,"/qc_p_fin/qc_sc_",id[pp],".txt"), quote = F,row.names = F)
}

names(flg_list) <- colnames(dattot)
totflgs_f4 <- sapply(flg_list, nrow)

######################################################################################
############# add the flags to the sef file and store in the daymeans qced folder
######################################################################################

## read flags from qc files
qcfiles <- list.files(path="code/data_prep/daymeans/sef/spatialqc/qc_p_fin/", full.names = T)
qcflags <- lapply(qcfiles, read.table, header=T)
names(qcflags) <- gsub(".txt","",gsub(".*qc_sc_","",qcfiles))

### apply qc functions to daily data again
metalist <- lapply(files, function(x){read_meta(paste0(filepath,x))})

for(ff in 1:length(allsefs)){
  print(names(allsefs)[[ff]])
  sf <- allsefs[[ff]]
  meta <- metalist[[ff]]
  metahead <- gsub("|QC software=dataresqc v1.0.3|QC software=dataresqc v1.1.0|","",meta["meta"])
  metahead <- paste0(stringr::str_sub(metahead, start = 1, end=-3),"QC software=dataresqc v1.1.0")
  newmeta <- sf[,"Meta"]
  f_ind <- which(names(qcflags) %in% id[ff])
  qc <- qcflags[[f_ind]]
  sefdates <- pdate(sf[,c("Year","Month","Day")])
  
  if(dim(qc)[1]>0){
    find <- which(sefdates %in% pdate(qc[,c("Year","Month","Day")]))
    newmeta[find] <- paste0(newmeta[find],ifelse(newmeta[find]=="","qc=spatial_consistency","|qc=spatial_consistency"))
    #sf$Value[find] <- NA
      }
  
  sefdata <- list()
  sefdata$p <- data.frame(Year = as.integer(sf[,"Year"]),
                          Month = as.integer(sf[,"Month"]),
                          Day = as.integer(sf[,"Day"]),
                          Hour = as.integer(sf[,"Hour"]),
                          Minute =as.integer(sf[,"Minute"]),
                          Value = sf[,"Value"],
                          Meta=newmeta)
  sefdata$p <- check_dates(sefdata$p)
  addinfo <-substr(gsub(".*_p","",id[ff]),2, nchar(gsub(".*_p","",id[ff])))
  addinfo <- paste0(addinfo,ifelse(nchar(addinfo)>0,"_",""))
  
  write_sef(Data = sefdata$p[, 1:7],
            outpath = "/scratch3/noemi/wear/swiss_arm/code/data_prep/daymeans/qced/",
            variable = "p",
            cod = meta["id"],
            nam = meta["name"],
            lat = meta["lat"],
            lon = meta["lon"],
            alt = meta["alt"],
            link = meta["link"],
            sou = meta["source"],
            units = meta["units"],
            stat = "mean",
            metaHead = metahead,
            meta=sefdata$p$Meta,
            period = "day",
            time_offset =0,
            keep_na = TRUE,
            note=paste0(addinfo,"qc_qcs"))
  
}

rm(list=ls())
