#### test the data for spatial consistency

## calculate regression between all time series for ta 

rm(list=ls())
plot <- F

require("dataresqc")
source("code/data_prep/scripts/C000_helpfun.R")
source("code/data_prep/scripts/C005_helpfun_ref.R")

outpath <- "code/data_prep/qc/spatialconsistency/"

## read files
filepath <- paste0(main,"code/data_prep/daymeans/qced/")
files <- list.files(filepath)
files <- files[grepl("_ta_", files)& !grepl("_qcs.", files)]

allsefs <- lapply(files, function(x){
  sef <- read_sef(paste0(filepath,x), all=T)
  return(sef)
  })

id <- gsub("_daily.tsv|_qc_daily.tsv","",files)
dates <- stringr::str_sub(gsub("_ta_daily.tsv","",files),start =-17)
names(allsefs) <- id 

## create matrix of all data
yr <- range(sapply(allsefs, function(x) range(x[,"Year"], na.rm=T)))
daterange <- seq(as.Date(paste0(yr[1],"-01-01")),as.Date(paste0("1863-12-31")),by="day")
dattot <- matrix(NA, ncol=length(allsefs),nrow=length(daterange))
colnames(dattot) <- id
dindm <- list()

for(ff in 1:length(allsefs)){
  y <- allsefs[[ff]]
  dindm[[ff]] <- daterange %in% pdate(y[,c("Year","Month","Day")])
  dinds<-  pdate(y[,c("Year","Month","Day")]) %in% daterange
  dattot[dindm[[ff]],ff] <- y$Value[dinds]
  }

## correlation between all
if(plot){
  corrmat <- cor(dattot, use="pairwise.complete.obs")
  rownames(corrmat) <- id
  png(paste0(outpath,"corrmatt_allsefs_ta.png"), width = 1000,height = 1000)
  cols <- rev(colorRampPalette(RColorBrewer::brewer.pal(n=21-1,"RdBu"))(21-1))
  corrplot::corrplot(corrmat, method = "square", na.label.col="gray55",col=cols)
  dev.off()
}

## linear models between all the datasets (sep into temp + pressure)
outmat <- array(NA, dim=dim(dattot))
estmat <- array(NA, dim=dim(dattot))
flginfo <- array(NA, dim=c(nrow(dattot),length(id),10))

tastns <- 1:length(id)
f <- 4

mods_ta <- list()
pred_ta <- list()

##########################
#### for temperature
##########################
dattot_deseas <- array(NA,dim(dattot))

seasonality <- F

# seasonality correction
if (seasonality) {
  print("filtering temperature seasonality")
  date <- daterange
  t_data <- dattot
  
  ### make vector x with doy/ndoy
  doy <- lubridate::yday(seq(as.Date(date[1]),as.Date(date[length(date)]), by = "day"))
  ndoy <- diff(seq(as.Date(date[1]), as.Date(paste((lubridate::year(as.Date(date[length(date)]))+1),"-01-01",sep="")), by = "year"))
  ndoy <- rep(ndoy,ndoy)
  ndoy <- ndoy[1:length(doy)]
  x <- doy/ndoy
  
  # define seasonality function
  rm.seas <- function(tempdata) {
    tempdata_cor <- tempdata
    
    temp.jg.cos <- cos(2*pi*x)
    temp.jg.sin <- sin(2*pi*x)
    temp.jg.cos2 <- cos(4*pi*x)
    temp.jg.sin2 <- sin(4*pi*x)
    
    temp.jg.lm <- lm(tempdata ~ temp.jg.cos + temp.jg.sin + temp.jg.cos2 + temp.jg.sin2)
    
    temp.jg.cor <- as.numeric(tempdata[as.numeric(names(temp.jg.lm$fitted.values))] - temp.jg.lm$fitted.values)
    tempdata_cor[!is.na(tempdata_cor)] <- temp.jg.cor
    return(tempdata_cor)
  }   
  
  # calculate seasonality-corrected temperature data
  deseas <- apply(t_data,2,rm.seas)
  stdsea<- apply(deseas,2,sd, na.rm=T)
  meansea<- apply(deseas,2,mean, na.rm=T)
  deseas_cntrd <- (deseas-meansea)/stdsea
  dattot_deseas[,colnames(dattot)%in%colnames(deseas)]<-deseas_cntrd
  rm(deseas,t_data,doy,ndoy,date,x,rm.seas)
}


##############################################

dattot_deseas <- dattot

for(ta in tastns){
  
  tadat <- dattot_deseas[,ta]
  #tadat <- dattot[,ta]
  
  mods_ta[[ta]] <- lapply(tastns[tastns!=ta],function(z){
    eqx <- sum(which(!is.na(tadat)) %in% which(!is.na(dattot_deseas[,z])))
    if(eqx>(365*5)){m<- lm(tadat~dattot_deseas[,z])} else{m <- NA}
    return(m)
  })
  
  pred_ta[[ta]] <- lapply(1:length(mods_ta[[ta]]),function(z){
    if(!is.na(mods_ta[[ta]][z])){
      m<- predict(mods_ta[[ta]][[z]])
    } else{
      m <- NA
    }
    return(m)
  })
  
  predmat <- matrix(NA, ncol=length(pred_ta[[ta]]), nrow=nrow(dattot))
  for (pd in 1:ncol(predmat)){
    ind <- as.integer(names(pred_ta[[ta]][[pd]]))
    predmat[ind,pd] <- pred_ta[[ta]][[pd]]
  }
  colnames(predmat) <- id[tastns[tastns!=ta]]
  ## standard error of each regression
  dathelp <- predmat-matrix(rep(tadat,ncol(predmat)), ncol=ncol(predmat))
  serr <- apply(dathelp,2,function(x){
    nna <- !is.na(x)
    return(sqrt(sum(x^2,na.rm=T)/sum(nna)))
  })
  rsquared <- sapply(mods_ta[[ta]],function(x) {if(!length(x)==1){summary(x)$r.squared}})
  ## select regressions with lowest standard errors, that are able to predict at the specific day of the time series
  ## i.e. loop through time series
  tadat_nna <- which(!is.na(tadat))
  for(day in tadat_nna){
    x <- dattot_deseas[day,ta]
    ## stations, that have a prediction for day
    pospred <- which(!is.na(dathelp[day,]))
    sp <- order(serr[pospred])
    len <- ifelse(length(sp)>4,5,length(sp))
    selreg <- pospred[sp][1:len]
    ## remove stations with errors larger than 5
    selreg <- selreg[!serr[selreg]>5]
    ## weighted standard errors of stations with lowest std error
    if(length(selreg)>1){
      sest <- sqrt(length(selreg)/sum(serr[selreg]^-2))
      ## create a weighted estimate of x' 
      sign <- sign(predmat[day,selreg])
      wvals <-  sum(sign*(predmat[day,selreg]^2)/(serr[selreg]^2))
      wsign <- sign(wvals)
      xest <- wsign*sqrt(abs(wvals))/sqrt(sum(serr[selreg]^-2))
      
      val <- x >= xest - sest*f & x <= xest + sest*f
      ## write to outmat if it is above,below ci
      outmat[day,ta] <- ifelse(val,0,1)
      estmat[day,ta] <- xest
      flginfo[day,ta,1:len] <- id[tastns[tastns!=ta]][selreg]
      flginfo[day,ta,6:(6+len-1)] <- round(serr[selreg],2)
    }
  }
}

## plot all time series and add in red, the values that have been flagged
flg_list <- list()
plot <- F

for(ta in tastns){
  ind <- range(which(!is.na(dattot[,ta])))
  
  if(plot){
    png(paste0(main,"/code/data_prep/qc/spatialconsistency/sctest_ta/sctest_ts_",id[ta],"_flagged_f",f,"_",Sys.Date(),".png"), height = 300, width = 1000,res=150)
    par(mar=c(2,2,1,1))
    plot(daterange[ind[1]:ind[2]],dattot[ind[1]:ind[2],ta], pch=19, cex=0.1, ylab="hPa",xlab="")
    colsel <- ifelse(outmat[ind[1]:ind[2],ta]==1,"red",NA)
    points(daterange[ind[1]:ind[2]],dattot[ind[1]:ind[2],ta], pch=19, cex=0.1, col=colsel)
    yrst <- substr(daterange[ind[1]:ind[2]],6,10)=="01-01"
    abline(v=daterange[ind[1]:ind[2]][yrst], lty=2, col="gray75")
    title(id[ta])
    dev.off()
  }

  flgind <- (ind[1]:ind[2])[outmat[ind[1]:ind[2],ta]==1]
  if(length(flgind)>0){
    yrs <- lubridate::year(daterange[flgind])
    mos <- lubridate::month(daterange[flgind])
    days <- lubridate::day(daterange[flgind])
    vals <- dattot[flgind,ta]
    stns <- cllps(flginfo[flgind,ta,1:5],"-")
    serrs <- cllps(flginfo[flgind,ta,6:10],"-")
    xest <- round(estmat[flgind,ta],2)
    sctest_flags <- data.frame(Var=rep("ta",length(yrs)),Year=yrs,Month=mos,Day=days,Value=vals,Estimate=xest,Refstns=stns,RSerrs=serrs)
    sctest_flags <- sctest_flags[!is.na(sctest_flags[,2]),]
    flg_list[[ta]] <- sctest_flags
    
  } else {
    sctest_flags <-data.frame(Var=NA,Year=NA,Month=NA,Day=NA,Value=NA,Estimate=NA,Refstns=NA,RSerrs=NA)
  }
  flg_list[[ta]] <- sctest_flags
  write.table(sctest_flags,file=paste0(main,"/code/data_prep/daymeans/sef/spatialqc/qc_ta_fin/qc_sc_ta_",id[ta],".txt"), quote = F,row.names = F)
}


names(flg_list) <- colnames(dattot)
# totflgs_f2 <- sapply(flg_list, nrow)
# totflgs_f4 <- sapply(flg_list, nrow)
totflgs_f3 <- sapply(flg_list, nrow)
# 
# ## the percentage change:
# percflgs_f2 <- totflgs_f2/apply(dattot,2,function(x) sum(is.na(x)))*100
# percflgs_f4 <- totflgs_f4/apply(dattot,2,function(x) sum(is.na(x)))*100
percflgs_f3 <- totflgs_f3/apply(dattot,2,function(x) sum(is.na(x)))*100

######################################################################################
############# add the flags to the sef file and store in the daymeans qced folder
######################################################################################

## read flags from qc files
qcfiles <- list.files(path="code/data_prep/daymeans/sef/spatialqc/qc_ta_fin/", full.names = T)
qcflags <- lapply(qcfiles, read.table, header=T)
names(qcflags) <- gsub(".txt","",gsub(".*qc_sc_ta_","",qcfiles))

### apply qc functions to daily data again
metalist <- lapply(files, function(x){read_meta(paste0(filepath,x))})

for(ff in 1:length(allsefs)){
  print(names(allsefs)[[ff]])
  sf <- allsefs[[ff]]
  meta <- metalist[[ff]]
  newmeta <- sf[,"Meta"]
  
  metahead <- gsub("|QC software=dataresqc v1.0.3|QC software=dataresqc v1.1.0|","",meta["meta"])
  metahead <- paste0(metahead,"QC software=dataresqc v1.1.0")
  f_ind <- which(names(qcflags) %in% id[ff])
  qc <- qcflags[[f_ind]]
  sefdates <- pdate(sf[,c("Year","Month","Day")])
  
  if(dim(qc)[1]>0){
    if(!all(is.na(qc[1,]))){
      find <- which(sefdates %in% pdate(qc[,c("Year","Month","Day")]))
      newmeta[find] <- paste0(newmeta[find],ifelse(newmeta[find]=="","qc=spatial_consistency","|qc=spatial_consistency"))
      #sf$Value[find] <- NA
      
      ## check the dates
      cdates <- seq(sefdates[1],sefdates[length(sefdates)], by="day")
      find2 <- which(!sefdates %in% cdates)
      newmeta[find2] <- paste0(newmeta[find2],ifelse(newmeta[find2]=="","qc=wrong_date","|qc=wrong_date"))
    }
  }
  
  sefdata <- list()
  sefdata$ta <- data.frame(Year = as.integer(sf[,"Year"]),
                          Month = as.integer(sf[,"Month"]),
                          Day = as.integer(sf[,"Day"]),
                          Hour = as.integer(sf[,"Hour"]),
                          Minute =as.integer(sf[,"Minute"]),
                          Value = sf[,"Value"],
                          Meta=newmeta)
  sefdata$ta <- check_dates(sefdata$ta)
  addinfo <-substr(gsub(".*_ta","",id[ff]),2, nchar(gsub(".*_ta","",id[ff])))
  addinfo <- paste0(addinfo,ifelse(nchar(addinfo)>0,"_",""))
  
  write_sef(Data = sefdata$ta[, 1:7],
            outpath = "/scratch3/noemi/wear/swiss_arm/code/data_prep/daymeans/qced/",
            variable = "ta",
            cod = meta["id"],
            nam = meta["name"],
            lat = meta["lat"],
            lon = meta["lon"],
            alt = meta["alt"],
            link = meta["link"],
            sou = meta["source"],
            units = meta["units"],
            stat = "mean",
            metaHead = metahead,
            meta=sefdata$ta$Meta,
            period = "day",
            time_offset =0,
            keep_na = TRUE,
            note=paste0(addinfo,"qc_qcs"))
  
}

rm(list=ls())

