#######################################################
############## create input for homogenization files
######################################################

require(maps)
require(ClimIndVis)
require(RColorBrewer)
require(dataresqc)

rm(list=ls())

source("code/data_prep/scripts/C000_helpfun.R")
source("code/data_prep/scripts/Cget_ncdf.R")

datdir <- "qced"
defyear <- 1890
precip_hom <- F
################################
## temp + pressure
################################

files <- list.files(paste0("code/data_prep/daymeans/",datdir), full.names = T)
files <- files[grepl("_qc_qcs.tsv", files)]
stndat <- lapply(files, function(s){return(read_sef(s,all=T))})
stnmeta <- sapply(files, function(s){return(read_meta(s))})
stnmeta <- t(stnmeta)
ff <-gsub("code/data_prep/daymeans/qced/","",files)
ff <- gsub("_qc_qcs.tsv","",ff)
id <- paste0(ff)
dates <- stringr::str_sub(gsub("_ta_qc_qcs.tsv|_p_qc_qcs.tsv","",files),start =-17)
rownames(stnmeta) <- id
names(stndat) <- id

## set value with flag to NA
stndat <- lapply(stndat, function(s){
  s[grepl("qc=",s$Meta),"Value"]<- NA
  return(s)})

## create matrix of all data
yr <- range(sapply(stndat, function(x) range(x[,"Year"])))
daterange <- seq(as.Date(paste0(yr[1],"-01-01")),as.Date(paste0(defyear,"-12-31")),by="day")
dattot <- matrix(NA, ncol=length(stndat),nrow=length(daterange))
colnames(dattot) <- as.character(id)
dindm <- list()

for(ff in 1:length(stndat)){
  y <- stndat[[ff]]
  dindm[[ff]] <- daterange %in% pdate(y[,c("Year","Month","Day")])
  dinds<-  pdate(y[,c("Year","Month","Day")]) %in% daterange
  dattot[dindm[[ff]],ff] <- y$Value[dinds]
}

climindvis <- make_object(tavg=t(dattot), dates_tavg = daterange,lon=stnmeta[,"lon"],lat=stnmeta[,"lat"], 
                          data_info = list(type="p",date_format="t1d",data_name="EIP"))
stnmons <- calc_index(climindvis, index="mean",var="tavg", aggt="monthly", NAmaxAgg=30)

for(ff in 1:length(stndat)){
  print(names(stndat)[ff])
  ## choose all years where there has been data, even if it was set to NA due to too many missing values
  seffyrs <- as.character(range(stndat[[ff]][,c("Year")]))
  if(seffyrs[2]> as.character(defyear)) {seffyrs[2] <- 1863}
  indr <- which(stnmons$index_info$years %in% seffyrs)
  datf <- data.frame(year=rep(as.integer(stnmons$index_info$years[indr[1]:indr[length(indr)]]),each=12),
                     mon= rep(c(1:12),length(stnmons$index_info$years[indr[1]:indr[length(indr)]])),
                     day=rep(00,length(rep(c(1:12),length(stnmons$index_info$years[indr[1]:indr[length(indr)]])))),
                     value= round(as.vector(stnmons$index[ff,,indr[1]:indr[length(indr)]]),1))
  write.table(datf,file=paste0("code/data_prep/hom_input/",id[ff],".dat"),quote=F,row.names = F,col.names = F)
}


################################
### precipitation
################################
if(precip_hom){
  files <- list.files(paste0("code/data_prep/daymeans/",datdir), full.names = T)
  files <- files[grepl("_rr_daily.tsv", files)]
  stndat <- lapply(files, function(s){return(read_sef(s,all=T))})
  stnmeta <- sapply(files, function(s){return(read_meta(s))})
  stnmeta <- t(stnmeta)
  id <- paste0(stnmeta[,"id"],"_",stnmeta[,"var"])
  rownames(stnmeta) <- id
  names(stndat) <- id
  
  ## set value with flag to NA
  stndat <- lapply(stndat, function(s){
    s[grepl("qc=",s$Meta),"Value"]<- NA
    return(s)})
  
  ## create matrix of all data
  yr <- range(sapply(stndat, function(x) range(x[,"Year"])))
  daterange <- seq(as.Date(paste0(yr[1],"-01-01")),as.Date(paste0(defyear,"-12-31")),by="day")
  dattot <- matrix(NA, ncol=length(stndat),nrow=length(daterange))
  colnames(dattot) <- as.character(id)
  dindm <- list()
  
  for(ff in 1:length(stndat)){
    y <- stndat[[ff]]
    dindm[[ff]] <- daterange %in% pdate(y[,c("Year","Month","Day")])
    dinds<-  pdate(y[,c("Year","Month","Day")]) %in% daterange
    dattot[dindm[[ff]],ff] <- y$Value[dinds]
  }
  
  climindvis <- make_object(prec = t(dattot), dates_prec = daterange,lon=stnmeta[,"lon"],lat=stnmeta[,"lat"], 
                            data_info = list(type="p",date_format="t1d",data_name="EIP"))
  stnmons <- calc_index(climindvis, index="sum",var="prec", aggt="monthly")
  
  for(ff in 1:length(stndat)){
    print(names(stndat)[ff])
    ## indr <- range(which(apply(stnmons$index[ff,,],2, function(x) !all(is.na(x)))))
    ## choose all years where there has been data, even if it was set to NA due to too many missing values
    seffyrs <- as.character(range(stndat[[ff]][,c("Year")]))
    if(seffyrs[2]> as.character(defyear)) {seffyrs[2] <- 1863}
    indr <- which(stnmons$index_info$years %in% seffyrs)
    
    datf <- data.frame(year=rep(as.integer(stnmons$index_info$years[indr[1]:indr[length(indr)]]),each=12),
                       mon= rep(c(1:12),length(stnmons$index_info$years[indr[1]:indr[length(indr)]])),
                       day=rep(00,length(rep(c(1:12),length(stnmons$index_info$years[indr[1]:indr[length(indr)]])))),
                       value= round(as.vector(stnmons$index[ff,,indr[1]:indr[length(indr)]]),1))
    write.table(datf,file=paste0("code/data_prep/hom_input/",id[ff],".dat"),quote=F,row.names = F,col.names = F)
  }
  
  
}

######################################################
###### create input reference series
######################################################

ekfdir <- "/scratch3/noemi/wear/swiss_arm/data/ekf400"
ekfiles <- list.files(ekfdir, full.names = T, pattern="v2.0.nc")
yrs <- as.integer(stringr::str_sub(ekfiles,start=-12, end =-9))
ekfiles <- ekfiles[yrs>=1755 & yrs < defyear]
yrs <- yrs[yrs>=1755 & yrs < defyear]

cutx=c(-20,20)
cuty=c(30,55)

### temperature
t2m <- list()
t2m$lon <-   get_ncdf(ekfiles[[1]],"air_temperature", sub=360, cutx=cutx,cuty=cuty)$lon
t2m$lat <-   get_ncdf(ekfiles[[1]],"air_temperature", sub=360, cutx=cutx,cuty=cuty)$lat
t2m$dat <- lapply(ekfiles, function(x){get_ncdf(x,"air_temperature", sub=360, cutx=cutx,cuty=cuty)$dat})
t2m$dat <- array(unlist(t2m$dat), dim=c(length(t2m$lon), length(t2m$lat),length(ekfiles)*12))
thelp <- seq(as.Date(paste0(yrs[1],"-01-01")),as.Date(paste0(yrs[length(yrs)],"-12-31")),by="day")
t2m$time <-thelp[which(lubridate::day(thelp)==15)] 

# fields::image.plot(t2m$lon,t2m$lat,t2m$dat[,,1])
# maps::map("world",add=T)
#46° 48' 4" N, 8° 13' 36" E.
mlat <- 46.8011
mlon <- 8.226667
t2m$ts <- t2m$dat[which.min(abs(t2m$lon-mlon)),which.min(abs(t2m$lat-mlat)),] 

datf <- data.frame(year=lubridate::year(t2m$time),
                   mon= lubridate::month(t2m$time),
                   day=rep(00,length(t2m$time)),
                   value= t2m$ts)

write.table(datf,file=paste0("code/data_prep/hom_input/EKF400_REF_ta.dat"),quote=F,row.names = F,col.names = F)
write.table(datf,file=paste0("code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_ta.dat"),quote=F,row.names = F,col.names = F)


### pressure
slp <- list()
slp$lon <-   get_ncdf(ekfiles[[1]],"air_pressure_at_sea_level", sub=360, cutx=cutx,cuty=cuty)$lon
slp$lat <-   get_ncdf(ekfiles[[1]],"air_pressure_at_sea_level", sub=360, cutx=cutx,cuty=cuty)$lat
slp$dat <- lapply(ekfiles, function(x){get_ncdf(x,"air_pressure_at_sea_level", sub=360, cutx=cutx,cuty=cuty)$dat})
slp$dat <- array(unlist(slp$dat), dim=c(length(slp$lon), length(slp$lat),length(ekfiles)*12))
thelp <- seq(as.Date(paste0(yrs[1],"-01-01")),as.Date(paste0(yrs[length(yrs)],"-12-31")),by="day")
slp$time <-thelp[which(lubridate::day(thelp)==15)] 

slp$dat <- slp$dat/100
# fields::image.plot(slp$lon,slp$lat,slp$dat[,,1])
# maps::map("world",add=T)
#46° 48' 4" N, 8° 13' 36" E.
mlat <- 46.8011
mlon <- 8.226667
slp$ts <- slp$dat[which.min(abs(slp$lon-mlon)),which.min(abs(slp$lat-mlat)),] 

datf <- data.frame(year=lubridate::year(slp$time),
                   mon= lubridate::month(slp$time),
                   day=rep(00,length(slp$time)),
                   value= slp$ts)

write.table(datf,file=paste0("code/data_prep/hom_input/EKF400_REF_p.dat"),quote=F,row.names = F,col.names = F)
write.table(datf,file=paste0("code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_p.dat"),quote=F,row.names = F,col.names = F)


### precipitation
rr <- list()
rr$lon <-   get_ncdf(ekfiles[[1]],"total\ precipitation", sub=360, cutx=cutx,cuty=cuty)$lon
rr$lat <-   get_ncdf(ekfiles[[1]],"total\ precipitation", sub=360, cutx=cutx,cuty=cuty)$lat
rr$dat <- lapply(ekfiles, function(x){get_ncdf(x,"total\ precipitation", sub=360, cutx=cutx,cuty=cuty)$dat})
rr$dat <- array(unlist(rr$dat), dim=c(length(rr$lon), length(rr$lat),length(ekfiles)*12))
thelp <- seq(as.Date(paste0(yrs[1],"-01-01")),as.Date(paste0(yrs[length(yrs)],"-12-31")),by="day")
rr$time <-thelp[which(lubridate::day(thelp)==15)] 

rr$dat <- rr$dat ## no conversion neeeded, because we need monthly precip sums
fields::image.plot(rr$lon,rr$lat,rr$dat[,,1])
maps::map("world",add=T)
#46° 48' 4" N, 8° 13' 36" E.
mlat <- 46.8011
mlon <- 8.226667
rr$ts <- rr$dat[which.min(abs(rr$lon-mlon)),which.min(abs(rr$lat-mlat)),] 

datf <- data.frame(year=lubridate::year(rr$time),
                   mon= lubridate::month(rr$time),
                   day=rep(00,length(rr$time)),
                   value= rr$ts)

write.table(datf,file=paste0("code/data_prep/hom_input/EKF400_REF_rr.dat"),quote=F,row.names = F,col.names = F)
rm(list=ls())