########################################################
############## create standard format for ref-series
########################################################
#  all to sef, including info in meta, where other datasets have been used

rm(list=ls())

require(dataresqc)
require(geosphere)
require(ncdf4)
require(XLConnect)
require(lubridate)

source("code/data_prep/scripts/C000_helpfun.R")
source("code/data_prep/scripts/Cget_ncdf.R")
source("code/data_prep/scripts/C005_helpfun_ref_new.R")

inventory <- XLConnect::readWorksheetFromFile("code/data_input/arm_stations_inventory_withnbcn.xlsx",sheet=1, header=T,colTypes="character",startRow=2)
refper <- c(1961,2020)
sef <- "code/data_prep/daymeans/hom/"
refpath <- "data/"
eobs_p <- "eobs/pp_ens_mean_3154052_v23.nc"
outpath <- "code/data_prep/daymeans/refseries/sef/"
temp_file <- list.files(path="code/data_prep/daymeans/refseries/sef","_ta_daily.tsv",full.names = T)

########################################
#### Druck
########################################
## eobs pressure is only converted to station altitude for the period 1961 to 2020.
## Thus, the qmapping is only done for this period.

varind <- which(inventory$var%in%c("p") & !grepl("no|ind|luc", inventory$no))
refstn <- inventory[varind,"reference_station_data"]
inventory[varind,"ID"]

msl <- c("MIL_p","PAD_p","TOR_p")
ff <- 6

for(ff in 1:length(varind)){
  print(ff)
  print(refstn[ff])
  ref <- stringr::str_sub(unlist(strsplit(refstn[ff],",")), end=3)
  
  if(!is.na(ref[1])){
    var <- inventory[varind[ff],"var"]
    id <- inventory[varind[ff],"ID"]
    xlat <- as.numeric(inventory[varind[ff],"lat_hist"])
    xlon <- as.numeric(inventory[varind[ff],"lon_hist"])
    alt  <- as.numeric(inventory[varind[ff],"alt_hist"])
    
       ## 1) if reference data is from mch
       if(ref=="mch"){
         ## these all stay at their station altitude level!
         refhelp <- unlist(strsplit(refstn[ff],","))
         dat <- read.table(paste0(refpath,refhelp), header = T, skip=2,na.strings = "-")
         meta <- read.table(paste0(refpath,gsub("data","legend",refstn[ff])),skip=31,nrow=1,
                            colClasses = rep("character",7),strip.white=T)
         ## https://rdrr.io/github/falling-fruit/fruitr/man/ch1903_to_wgs84.html
         ch <- fruitr::ch1903_to_wgs84(as.integer(c(substr(meta["V6"],1,6),substr(meta["V6"],8,14))))
         xlon <- ch[1]
         xlat <- ch[2]
         alt <- as.numeric(meta["V7"])
         
         out <- data.frame(Year=as.integer(substr(dat$time,1,4)),
                           Month=as.integer(substr(dat$time,5,6)),
                           Day=as.integer(substr(dat$time,7,8)),
                           Hour=rep(NA, nrow(dat)),
                           Minute=rep(NA, nrow(dat)),
                           Value=as.numeric(dat[,"prestad0"]),
                           Meta=rep("mchstn",nrow(dat)), stringsAsFactors = F)
         out <- check_dates(out)
         refind <- which(out$Year %in% c(refper[1]:refper[2]))
         out <- out[refind,]
         
         write_sef(out[,1:7], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), meta["V1"], 
                   xlat, xlon, alt, "MCH", "idaweb", units="hPa", stat="mean",metaHead = "SLP=T", period="day", meta = out$Meta, note="daily", keep_na=TRUE)
         
         ## make a qc
         file <- list.files(outpath, pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
         qc(file, outpath=paste0("code/data_prep/daymeans/refseries/qcfiles/"))
         qcf <- list.files(paste0("code/data_prep/daymeans/refseries/qcfiles/"), pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
         if(length(qcf)>0){
           write_flags(file, qcfile=qcf,
                       outpath=outpath)
         }
         if(inventory[varind[ff],"gap_filling_data"]!="no"){
           print("fill gaps")
           out <- read_sef(file, all = T)
           out[grepl("qc=",out$Meta),"Value"] <- NA
           newout <- check_dates(out[,2:ncol(out)])
           newout <- cbind(rep("p", nrow(newout)), newout)
           out <- fill_gaps(newout,gap_data=inventory[varind[ff],"gap_filling_data"],xlon=xlon,xlat=xlat,var=var,id=id) 
           write_sef(out[,c("Year","Month","Day","Hour","Minute","Value")], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), meta["V1"], 
                     xlat, xlon, alt, "MCH", "idaweb", units="hPa", stat="mean",metaHead = "SLP=T", period="day", meta = out$Meta, note="daily", keep_na=TRUE)
                  
         }
         
       }
       ## 2) if reference data is from eobs
       if(ref=="eob"){
         out <- read_eobs(path=get(paste0("eobs_",var)),xlon=xlon,xlat=xlat,var=var)
         out[,2:ncol(out)] <- check_dates(out[,2:ncol(out)])
         refind <- which(out$Year %in% c(refper[1]:refper[2]))
         out <- out[refind,]
         if(inventory[varind[ff],"fin_plevel"] =="station"){
           temp_dat <- read_sef(temp_file[grepl(paste0(gsub("p","ta_ref",id)),temp_file)],all=T)
           out[,2:ncol(out)] <- convert_to_stn_altitude(out=out[,2:ncol(out)], alt=alt, id=id,temp_dat=temp_dat)
         }
         
         write_sef(out[,c("Year","Month","Day","Hour","Minute","Value")], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), paste0("eobs_",var), 
                   xlat, xlon, inventory[varind[ff],"alt_hist"], sou="EOBS",
                   link="http://surfobs.climate.copernicus.eu/dataaccess/access_eobs.php", 
                   units="hPa", stat="mean", meta=rep("closest_gp_eobs", length(refind)),
                   period="day",  note="daily", keep_na=TRUE)
       }
       ## 3) if reference data is from ecad
       if(ref=="ECA"){
         
         dat <- read.table(paste0(refpath,"ecad/",refstn[ff]), header = T, skip=20, sep=",",na.strings=-9999)
         meta <- read.table(paste0(refpath,"ecad/",gsub("/.*","",refstn[ff]),"/stations.txt"),
                            skip=19,sep=",",colClasses = rep("character",6), strip.white=T, allowEscapes=T)
         meta <- meta[which(meta$V1==dat$STAID[1]),]
         xlon  <- as.numeric(substr(meta["V5"],2,4))+as.numeric(substr(meta["V5"],6,7))/60+as.numeric(substr(meta["V5"],9,10))/3600
         xlat  <- as.numeric(substr(meta["V4"],2,3))+as.numeric(substr(meta["V4"],5,6))/60+as.numeric(substr(meta["V4"],8,9))/3600
         realend <- which(sapply(1:nrow(dat),function(i) all(is.na(dat[i:nrow(dat),4]))))[1]
         if (!is.na(realend)) dat <- dat[1:(realend-1),]
         
         out <- data.frame(Year=as.integer(substr(dat$DATE,1,4)),
                           Month=as.integer(substr(dat$DATE,5,6)),
                           Day=as.integer(substr(dat$DATE,7,8)),
                           Hour=rep(NA, nrow(dat)),
                           Minute=rep(NA, nrow(dat)),
                           Value=dat[,"PP"]*0.1,
                           Meta=rep("ECAD",nrow(dat)), stringsAsFactors = F)
         out$Value[which(dat[,"Q_PP"] %in% c(1,9))] <- NA
         out <- check_dates(out)
         refind <- which(out$Year %in% c(refper[1]:refper[2]))
         out <- out[refind,]
         if(inventory[varind[ff],"fin_plevel"] =="station"){
           temp_dat <- read_sef(temp_file[grepl(paste0(gsub("p","ta_ref",id)),temp_file)],all=T)
           out <- convert_to_stn_altitude(out=out, alt=alt, id=id,temp_dat=temp_dat)
         }
         
         write_sef(out[,1:7], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), meta["V2"], 
                   lat=xlat, lon=xlon, meta["V6"], sou="ECAD", link="",
                   units ="hPa", stat="mean", period="day", metaHead = "SLP=T",meta=out$Meta, note="daily", keep_na=TRUE)
         
         
         ## make a qc
         file <- list.files(outpath, pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
         qc(file, outpath=paste0("code/data_prep/daymeans/refseries/qcfiles/"))
         qcf <- list.files(paste0("code/data_prep/daymeans/refseries/qcfiles/"), pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
         if(length(qcf)>0){
           write_flags(file, qcfile=qcf,
                       outpath=outpath)
         }
         if(inventory[varind[ff],"gap_filling_data"]!="no"){
           print("fill gaps")
           out <- read_sef(file, all = T)
           out[grepl("qc=",out$Meta),"Value"] <- NA
           newout <- check_dates(out[,2:ncol(out)])
           newout <- cbind(rep("p", nrow(newout)), newout)
           out <- fill_gaps(newout,gap_data=inventory[varind[ff],"gap_filling_data"],xlon=xlon,xlat=xlat,var=var,id=id) 
              write_sef(out[,c("Year","Month","Day","Hour","Minute","Value")], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), meta["V2"], 
                     lat=xlat, lon=xlon, meta["V6"], sou="ECAD", link="",
                     units ="hPa", stat="mean", period="day", metaHead = "SLP=T",meta=out$Meta, note="daily", keep_na=TRUE)
           
         }
         
       }
       ## 4) if reference data is from ecad
       if(ref=="Wea"){
         refhelp <- unlist(strsplit(refstn[ff],","))
         dat <- read_sef(paste0("code/data_prep/daymeans/qced/",refhelp), all=T)
         meta <- read_meta(paste0("code/data_prep/daymeans/qced/",refhelp))
         out <- data.frame(Year=dat$Year,
                           Month=dat$Month,
                           Day=dat$Day,
                           Hour=dat$Hour,
                           Minute=dat$Minute,
                           Value=dat$Value,
                           Meta=dat$Meta, stringsAsFactors = F)
         out <- check_dates(out)
         # temp_dat <- read_sef(temp_file[grepl(paste0(gsub("p","ta_ref",id)),temp_file)],all=T)
         # out <- convert_to_sealevel(out=out, alt=alt, id=id,temp_dat=temp_dat)
         # 
         refind <- which(out$Year %in% c(refper[1]:refper[2]))
         out <- out[refind,]
         
         write_sef(out[,1:7], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), meta["name"], 
                   lat=meta["lat"], lon=meta["lon"], meta["alt"], sou="WeaR", link="",
                  units ="hPa", stat="mean", period="day", metaHead = "",meta=out$Meta, note="daily", keep_na=TRUE)
         
         ## make a qc
         file <- list.files(outpath, pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
         qc(file, outpath=paste0("code/data_prep/daymeans/refseries/qcfiles/"))
         qcf <- list.files(paste0("code/data_prep/daymeans/refseries/qcfiles/"), pattern = paste0(inventory[varind[ff],"ID"],"_ref"), full.names = T)
         if(length(qcf)>0){
           write_flags(file, qcfile=qcf,
                       outpath=outpath)
         }
         if(inventory[varind[ff],"gap_filling_data"]!="no"){
           print("fill gaps")
           out <- read_sef(file, all = T)
           out[grepl("qc=",out$Meta),"Value"] <- NA
           newout <- check_dates(out[,2:ncol(out)])
           newout <- cbind(rep("p", nrow(newout)), newout)
           out <- fill_gaps(newout,gap_data=inventory[varind[ff],"gap_filling_data"],xlon=xlon,xlat=xlat,var=var,id=id) 
           write_sef(out[,c("Year","Month","Day","Hour","Minute","Value")], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), meta["name"], 
                     lat=meta["lat"], lon=meta["lon"], meta["alt"], sou="WeaR", link="",
                     units ="hPa", stat="mean", period="day", metaHead = "",meta=out$Meta, note="daily", keep_na=TRUE)
           
         }
         
       }

  } 
}


#### plot all reference series for temp
seffiles_p <- list.files(path="code/data_prep/daymeans/refseries/sef/","_p_daily", full.names = T)
seffiles_p <- seffiles_p[!grepl("qc_",seffiles_p)]
sef <- lapply(seffiles_p, read_sef, all=T)
sef <- lapply(sef, function(x) { 
  if(any(grepl("qc=",x$Meta))){x$Value[grepl("qc=",x$Meta)] <- NA}
  return(x)
  })
sefnames <- gsub(".*/","",seffiles_p)
names(sef) <- sefnames

## Control plots
png(paste0("code/data_prep/plots/refseries/p/all_series_p_",Sys.Date(),".png"), width = 2000, height=4500, res=160)
par(mfrow=c(length(seffiles_p),1), mar=c(2,2,1,1))
for(st in 1:length(seffiles_p)){
  cols <- ifelse(grepl("pred.val",sef[[st]]$Meta),"forestgreen","black")
  plot(pdate(sef[[st]][,2:4]),sef[[st]][,"Value"], type="p",pch=19,cex=0.4, col=cols, xlim=as.Date(c("1961-01-01","2020-12-31")))
  mtext(side=3, sefnames[st])
}
dev.off()

sef <- lapply(sef, function(x){
  print("x")
  doys <- lubridate::yday(pdate(x[,2:4]))
  doymeans <- aggregate(as.numeric(x$Value), by=list(doys), "mean", na.rm=T)
  x$dailyanoms <- x$Value - doymeans$x[doys]
  return(x)  
})

png(paste0("code/data_prep/plots/refseries/p/all_series_dailyanoms_p_",Sys.Date(),".png"), width = 2000, height=4500, res=160)
par(mfrow=c(length(seffiles_p),1), mar=c(2,2,1,1))
for(st in 1:length(seffiles_p)){
  cols <- ifelse(grepl("pred.val",sef[[st]]$Meta),"forestgreen","black")
  plot(pdate(sef[[st]][,2:4]),sef[[st]][,"dailyanoms"], type="p",pch=19,cex=0.4, col=cols, xlim=as.Date(c("1961-01-01","2020-12-31")))
  mtext(side=3, sefnames[st])
}
dev.off()


rm(list=ls())
