########################################################
############## create standard format for ref-series
########################################################

rm(list=ls())

require(dataresqc)
require(geosphere)
require(ncdf4)
require(XLConnect)
require(lubridate)
require(qmap)

source("code/data_prep/scripts/Cget_ncdf.R")
source("code/data_prep/scripts/C005_helpfun_ref.R")
source("code/data_prep/scripts/C000_helpfun.R")

inventory <- XLConnect::readWorksheetFromFile("code/data_input/arm_stations_inventory_withnbcn.xlsx",sheet=1, header=T,colTypes="character",startRow=2)
refper <- c(1961,2020)
sef <- "code/data_prep/daymeans/hom/"
refpath <- "data/"
rhires <- "code/data_input/RhiresD_ch01h.swiss.lv95_19610101-20201231.nc"
outpath <- "code/data_prep/daymeans/refseries/hom/"

########################################
#### Precip
########################################
varind <- which(inventory$var%in%c("rr") & inventory$no!="no")
refstn <- inventory[varind,"reference_station_data"]
inventory[varind,"ID"]

for(ff in 1:length(varind)){
  
  print(refstn[ff])
  (ref <- stringr::str_sub(unlist(strsplit(refstn[ff],",")), end=3))
  var <- inventory[varind[ff],"var"]
  id <- inventory[varind[ff],"ID"]
  xlat <- as.numeric(inventory[varind[ff],"lat_hist"])
  xlon <- as.numeric(inventory[varind[ff],"lon_hist"])
  
  ## 1) if reference data is from mch
  if(ref=="mch"){
    refhelp <- unlist(strsplit(refstn[ff],","))
    dat <- read.table(paste0(refpath,refhelp), header = T, skip=2,na.strings = "-", sep=";")
    dat <- dat[dat$stn==gsub("_rr","",id),]
    meta <- read.table(paste0(refpath,gsub("data","legend",refstn[ff])),skip=31,nrow=6,
                       colClasses = rep("character",7),strip.white=T)
    meta <- meta[meta$V1==gsub("_rr","",id),]
    
    ## https://rdrr.io/github/falling-fruit/fruitr/man/ch1903_to_wgs84.html
    ch <- fruitr::ch1903_to_wgs84(as.integer(c(substr(meta["V6"],1,6),substr(meta["V6"],8,14))))
    xlon <- ch[1]
    xlat <- ch[2]
    
    out <- data.frame(Year=as.integer(substr(dat$time,1,4)),
                      Month=as.integer(substr(dat$time,5,6)),
                      Day=as.integer(substr(dat$time,7,8)),
                      Hour=rep(NA, nrow(dat)),
                      Minute=rep(NA, nrow(dat)),
                      Value=as.numeric(as.character(dat[,"rhs150d0"])),
                      Meta=rep("mchstn",nrow(dat)))
    out <- check_dates(out)
    refind <- which(out$Year %in% c(refper[1]:refper[2]))
    out <- out[refind,]
    
    write_sef(out[,1:7], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), meta["V1"], 
              xlat, xlon, meta["V7"], "WeaR", "idaweb", units="mm", "sumn", period="day", meta = out$Meta, note="hom", keep_na=TRUE)
    
  }
  ## 2) if reference from rhires
  if(ref=="rhi"){
    dat <- list()
    ncp <- nc_open(rhires)
    lons <- ncvar_get(ncp, "lon")
    lats <- ncvar_get(ncp, "lat")
    coords <- data.frame(lon=as.vector(lons), lat=as.vector(lats))
    
    ## get coordinates of station
    xlon <- as.numeric(inventory[varind[ff],"lon_hist"])
    xlat <- as.numeric(inventory[varind[ff],"lat_hist"])
    
    ## Select closest grid point on land (limit the search to a square of 25 grid points)       
    i_grid <- which.min(distGeo(c(xlon,xlat), coords))
    i_lonlat <- which(lons == coords$lon[i_grid], arr.ind = T)
    dat$dat <- ncvar_get(ncp, "RhiresD", c(i_lonlat[1],i_lonlat[2],1), c(1,1,-1))
    dat$time <- nc_time_p(ncp,timename = "time")
    
    out <- data.frame(Year=lubridate::year(dat$time),
                      Month=lubridate::month(dat$time),
                      Day=lubridate::day(dat$time),
                      Hour=rep(NA, nrow(dat$dat)),
                      Minute=rep(NA, nrow(dat$dat)),
                      Value=round(dat$dat,2),
                      Meta=rep("RhiresD",nrow(dat$dat)))
    out <- check_dates(out)
    refind <- which(out$Year %in% c(refper[1]:refper[2]))
    out <- out[refind,]
    
    write_sef(out[,1:7], outpath, var, paste0(inventory[varind[ff],"ID"],"_ref"), inventory[varind[ff],"Name"], 
              lat=xlat, lon=xlon, inventory[varind[ff],"alt_hist"], sou="WeaR", link="",
              units ="mm", stat="sum", period="day", metaHead = "",meta=out$Meta, note="hom", keep_na=TRUE)
      }
}

#### do a qc just as a test.
rrfiles <- list.files(pattern="_rr_",path=outpath,full.names = T)
rrfiles <- rrfiles[!grepl("qc_",rrfiles)]
lapply(rrfiles,function(x) {
  print(x)
  qc(x, outpath = outpath) 
})
# 
flgfiles <- list.files(pattern="_rr_ref_rr_",path=outpath,full.names = T)



#### plot all reference series for temp
seffiles_rr <- list.files(path="code/data_prep/daymeans/refseries","_rr_daily", full.names = T)
seffiles_rr <- seffiles_rr[!grepl("KAR|qc_",seffiles_rr)] 
sef <- lapply(seffiles_rr, read_sef, all=T)
sef <- lapply(sef, function(x) { 
  if(any(grepl("qc=",x$Meta))){x$Value[grepl("qc=",x$Meta)] <- NA}
  return(x)
})
sefnames <- gsub(".*/","",seffiles_rr)
names(sef) <- sefnames

## Control plots
png(paste0("code/data_prep/plots/refseries/rr/all_series_rr_",Sys.Date(),".png"), width = 2000, height=3000, res=160)
par(mfrow=c(length(seffiles_rr),1), mar=c(2,2,1,1))
for(st in 1:length(seffiles_rr)){
  cols <- ifelse(grepl("pred.val",sef[[st]]$Meta),"forestgreen","black")
  plot(pdate(sef[[st]][,2:4]),sef[[st]][,"Value"], type="p",pch=19,cex=0.4, col=cols, xlim=as.Date(c("1961-01-01","2020-12-31")))
  mtext(side=3, sefnames[st])
}
dev.off()
