###############################################
############## Merge long series
##############################################

#### visualize all series
#### correlations
#### are there annual cycles of differences?
#### compare distributions
#### breakpoints

rm(list=ls())

require(maps)
require(ClimIndVis)
require(RColorBrewer)
require(dataresqc)
require(lubridate)

source('code/data_prep/scripts/C000_helpfun.R')
source("code/data_prep/scripts/C005_helpfun_ref.R")
source("code/data_prep/RHtests-master/RHtests-master/V4_files/RHtestsV4_20190301.r")

### path
homref <- "code/data_prep/daymeans/histseries/hom/"
finpath <- "code/data_prep/daymeans/histseries/merged/"
plotpath <- "code/data_prep/plots/bern/"

### data
seffiles <- list.files(homref, full.names=T, pattern="GCOS_Bern|Sutz|Burgdorf|Gurzelen|Bueren")
id <- sapply(seffiles, function(x){read_meta(x)["id"]})
id_full <- gsub("_hom.tsv|_not_homogenized.tsv","",gsub(".*/","",seffiles))
nam <- sapply(seffiles, function(x){read_meta(x)["name"]})
varf <- sapply(seffiles, function(x){read_meta(x)["var"]})
lats <- sapply(seffiles, function(x){read_meta(x)["lat"]})
lons <- sapply(seffiles, function(x){read_meta(x)["lon"]})
alts <- as.numeric(sapply(seffiles, function(x){read_meta(x)["alt"]}))
allsefs <- lapply(seffiles, function(x){
  sef <- read_sef(x, all=T)
  sef$Value[grepl("qc=",sef$Meta)] <- NA
  return(sef)
})
names(allsefs) <- paste0(id_full)
names(alts) <- paste0(id_full)
pind <- which(!varf=="ta")

## read in the current climate bern p file
bern_ref <- read_sef("code/data_prep/daymeans/refseries/hom/WeaR_BER_p_ref_19610101-20200630_p_nothom.tsv")
## clim
bern_yclim <- aggregate(bern_ref$Value, list(yday(pdate(bern_ref[,2:4]))), function(x){quantile(x, probs=c(0.02,0.25,0.5,0.75,0.98), na.rm=T)})

## create matrix of all data
yr <- range(sapply(allsefs[pind], function(x) range(x[,"Year"])))
daterange <- seq(as.Date(paste0(yr[1],"-01-01")),as.Date(paste0("1863-12-31")),by="day")
dattot <- data.frame(matrix(NA, ncol=length(pind)+4,nrow=length(daterange)))
colnames(dattot) <- c("year","mon","day",as.character(id_full[pind]),"Meta")
dindm <- list()

for(ff in 1:length(pind)){
  y <- allsefs[[pind[ff]]]
  dindm[[ff]] <- daterange %in% pdate(y[,c("Year","Month","Day")])
  dinds<-  pdate(y[,c("Year","Month","Day")]) %in% daterange
  dattot[dindm[[ff]],ff+3] <- y$Value[dinds]
}
dattot[,c("year","mon","day")] <- cbind(year(daterange),month(daterange),day(daterange))
dattot[,c("year","mon","day")] <- cbind(year(daterange),month(daterange),day(daterange))

doymeans <- matrix(unlist(apply(dattot[,4:(ncol(dattot)-1)],2, function(x){aggregate(x,list(yday_without_29feb(daterange)), mean, na.rm=T)$x})), nrow=365,ncol=8)
dattotanoms <-dattot[,4:(ncol(dattot)-1)] - doymeans[yday_without_29feb(daterange),]
corrmat2 <- round(cor(dattotanoms, use="pairwise.complete.obs"),2)

png(paste0(plotpath,"bern_cormat_panoms.png"), width = 1200, height = 1200, res=180)
par(mar=c(12,12,2,2))
cols <- colorRampPalette(RColorBrewer::brewer.pal(n=11-1,"Reds"))(11-1)
fields::image.plot(1:ncol(corrmat2),1:ncol(corrmat2),corrmat2, col=cols, breaks=seq(0,1,0.1),ylab="",xlab="", xaxt="n", yaxt="n")
axis(side=1, las=2, labels = id[pind], at=seq(1:ncol(corrmat2)))
axis(side=2, las=2,labels = id[pind], at=seq(1:ncol(corrmat2)))
abline(v=1:ncol(corrmat2), lty=2, col="gray65", lwd=0.5)
abline(h=1:ncol(corrmat2), lty=2, col="gray65", lwd=0.5)
dev.off()


## annual distr plots
distr <- F
if(distr){
  dist_all_ts <- lapply(1:length(pind), function(y){
    datstn <- dattot[,y+3]
    dist <- t(sapply(1:365, function(x){quant_window_fun(dat=datstn,dates=daterange,doy=x)} ))
    return(dist)
  })
  
  names(dist_all_ts) <- colnames(dattot[4:(ncol(dattot)-1)])
  
  png(paste0(plotpath,"annual_cycle_tavel_trechsel_lombach_p_",Sys.Date(),".png"), width = 1000)
  ll <-1.5
  par(mfrow=c(1,1), mar=c(2,2,1,2))
  plot(1:365,dist_all_ts$BE01_Bern_Tavel[,3], type="l", ylim=c(930,970), col="turquoise1", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Trechsel[,3], type="l", col="darkred", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Lombach[,3], type="l", col="forestgreen", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Lombach[,5], type="l", col="forestgreen", lwd=ll, lty=3)
  points(1:365,dist_all_ts$BE01_Bern_Tavel[,5], type="l", lty=3, col="turquoise1", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Trechsel[,5], type="l", lty=3, col="darkred", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Tavel[,1], type="l", lty=3, col="turquoise1", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Trechsel[,1], type="l", lty=3,col="darkred", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Lombach[,1], type="l", col="forestgreen", lwd=ll, lty=3)
  abline(h=seq(900,990,10),lty=3, col="gray65")
  legend("bottom", legend=c("Tavel median","Tavel q10/q90","Trechsel","Trechsel q10/q90","Lombach","Lombach q10/q90"),  
         col=c("turquoise1","turquoise1","darkred","darkred","forestgreen","forestgreen"),lty = c(1,3,1,3,1,3),bty="n", cex=1)
  title("Jahresgang unterschiedl.Perioden")
  dev.off()
  
  png(paste0(plotpath,"annual_cycle_gurzelen_trechsel_lombach_p_",Sys.Date(),".png"), width = 1000)
  ll <-1.5
  par(mfrow=c(1,1), mar=c(2,2,1,2))
  plot(1:365,dist_all_ts$GCOS_Gurzelen[,3], type="l", ylim=c(930,970), col="turquoise1", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Trechsel[,3], type="l", col="darkred", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Lombach[,3], type="l", col="forestgreen", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Lombach[,5], type="l", col="forestgreen", lwd=ll, lty=3)
  points(1:365,dist_all_ts$GCOS_Gurzelen[,5], type="l", lty=3, col="turquoise1", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Trechsel[,5], type="l", lty=3, col="darkred", lwd=ll)
  points(1:365,dist_all_ts$GCOS_Gurzelen[,1], type="l", lty=3, col="turquoise1", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Trechsel[,1], type="l", lty=3,col="darkred", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Lombach[,1], type="l", col="forestgreen", lwd=ll, lty=3)
  abline(h=seq(900,990,10),lty=3, col="gray65")
  legend("bottom", legend=c("Gurzelen","Trechsel","Lombach"),  
         col=c("turquoise1","turquoise1","darkred","darkred","forestgreen","forestgreen"),lty = c(1,3,1,3,1,3),bty="n", cex=1)
  title("Jahresgang unterschiedl.Perioden")
  dev.off()
  
  
  png(paste0(plotpath,"annual_cycle_bueren_trechsel_lombach_p_",Sys.Date(),".png"), width = 1000)
  ll <-1.5
  par(mfrow=c(1,1), mar=c(2,2,1,2))
  plot(1:365,dist_all_ts$BE05_Bueren[,3], type="l", ylim=c(930,990), col="turquoise1", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Trechsel[,3], type="l", col="darkred", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Lombach[,3], type="l", col="forestgreen", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Lombach[,5], type="l", col="forestgreen", lwd=ll, lty=3)
  points(1:365,dist_all_ts$BE05_Bueren[,5], type="l", lty=3, col="turquoise1", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Trechsel[,5], type="l", lty=3, col="darkred", lwd=ll)
  points(1:365,dist_all_ts$BE05_Bueren[,1], type="l", lty=3, col="turquoise1", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Trechsel[,1], type="l", lty=3,col="darkred", lwd=ll)
  points(1:365,dist_all_ts$BE01_Bern_Lombach[,1], type="l", col="forestgreen", lwd=ll, lty=3)
  abline(h=seq(900,990,10),lty=3, col="gray65")
  legend("bottom", legend=c("Bueren median","Bueren q10/q90","Trechsel","Trechsel q10/q90","Lombach","Lombach q10/q90"),  
         col=c("turquoise1","turquoise1","darkred","darkred","forestgreen","forestgreen"),lty = c(1,3,1,3,1,3),bty="n", cex=1)
  title("Jahresgang unterschiedl.Perioden")
  dev.off()
  
}

### plot for every year all data points #####
redblues <- colorRampPalette(c("darkgreen","forestgreen","yellowgreen","yellow","cornflowerblue","blue","purple","magenta"))
stncols <- redblues(length(pind))

for (yr in unique(dattot[,"year"])){
  yrind <- dattot[,"year"]==yr
  print(yr)
  stns <- which(apply(dattot[yrind,4:ncol(dattot)],2, function(x){!all(is.na(x))}))
  if(length(stns)>0){
    png(paste0(plotpath,"years_p/dailydata_bern_yr",yr,"_",Sys.Date(),".png"), width =1000, height=500)
    dr <- range(dattot[yrind,stns+3], na.rm=T)
    plot(NA, xlim=range(pdate(dattot[yrind,c(1:3)])),col=stncols[stns[1]], pch=19, ylab="",xlab=yr, ylim=dr + c(-3,3), cex=0.7, xaxt="n")
    axis(side=1, at=pdate(dattot[yrind,c(1:3)])[which(substr(pdate(dattot[yrind,c(1:3)]),8,10)=="-01")], pdate(dattot[yrind,c(1:3)])[which(substr(pdate(dattot[yrind,c(1:3)]),8,10)=="-01")])
    ydays <- yday(pdate(dattot[yrind,c(1:3)]))
    polygon(c(pdate(dattot[yrind,c(1:3)]), rev(pdate(dattot[yrind,c(1:3)]))), 
            c(caTools::runmean(bern_yclim$x[ydays,1], k=10), caTools::runmean(rev(bern_yclim$x[ydays,5]),10)), col=t_col("gray85",50), border=NA)
    polygon(c(pdate(dattot[yrind,c(1:3)]), rev(pdate(dattot[yrind,c(1:3)]))), 
            c(caTools::runmean(bern_yclim$x[ydays,2], k=10), caTools::runmean(rev(bern_yclim$x[ydays,4]),10)), col=t_col("gray75",50), border=NA)
    
    if(length(stns)>0){
      for(ss in 1:length(stns)){
        points(pdate(dattot[yrind,c(1:3)]), dattot[yrind, stns[ss]+3], col=stncols[stns[ss]], pch=19, cex=0.7)
        points(pdate(dattot[yrind,c(1:3)]), ifelse(is.na(dattot[yrind, stns[ss]+3]),dr[2] + c(ss/2) ,NA), col=stncols[stns[ss]], pch=8, cex=0.7)
      }
    }
    legend("bottom",  col = stncols[stns], legend = id_full[pind][stns], pch=19, bty="n")
    dev.off()
  }
}

# #### merge the datasets #####
range(allsefs$BE01_Bern_Tavel_p$Year)
range(allsefs$BE01_Bern_Reinhard_p$Year)
yrr <- c(1760,1863)
daterange <- seq(as.Date(paste0(yrr[1],"-01-01")),as.Date(paste0(yrr[2],"-12-31")), by="day")
sefdata <- data.frame(year=year(daterange),
                      mo=month(daterange),
                      d=day(daterange),
                      h=rep(NA, length(daterange)),
                      m=rep(NA,length(daterange)),
                      Value=rep(NA,length(daterange)),
                      Meta=rep(NA,length(daterange)))

### add Tavel
tavind <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Tavel_17600101-17660630_p"]][,c(2:4)]) )
sefdata[tavind,"Value"] <- allsefs[["GCOS_Bern_Tavel_17600101-17660630_p"]]$Value
sefdata[tavind,"Meta"] <- "GCOS_Bern_Tavel_17600101-17660630_p"
### add Lombach
lomind <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Lombach_17770101-17850321_p"]][,c(2:4)]) )
sefdata[lomind,"Value"] <- allsefs[["GCOS_Bern_Lombach_17770101-17850321_p"]]$Value
sefdata[lomind,"Meta"] <- "GCOS_Bern_Lombach_17770101-17850321_p"
lomind2 <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Lombach_17850616-17891231_p"]][,c(2:4)]) )
sefdata[lomind2,"Value"] <- allsefs[["GCOS_Bern_Lombach_17850616-17891231_p"]]$Value
sefdata[lomind2,"Meta"] <- "GCOS_Bern_Lombach_17850616-17891231_p"
### add Studer where Lombach is missing
studind <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Studer_17791220-17891130_p"]][,c(2:4)]) & is.na(sefdata$Value))
helpind <- which(pdate(allsefs[["GCOS_Bern_Studer_17791220-17891130_p"]][,c(2:4)]) %in% daterange[studind])
sefdata[studind,"Value"] <- allsefs[["GCOS_Bern_Studer_17791220-17891130_p"]]$Value[helpind]
sefdata[studind,"Meta"] <- "GCOS_Bern_Studer_17791220-17891130_p"
### add Bueren
temp_bueren <- read_sef(paste0(homref,"/GCOS_Bueren_17891220-17950503_ta_not_homogenized.tsv"), all=T)
buerdat <- convert_to_stn_altitude(out=allsefs$`GCOS_Bueren_17891220-17970106_p`[,2:9], temp_dat=temp_bueren,
                                alt=alts["GCOS_Bern_Studer_17791220-17891130_p"]-alts["GCOS_Bueren_17891220-17970106_p"], id=NA)
burind <- which(daterange %in% pdate(buerdat[,1:3]))
helpind <- which(pdate(buerdat[,1:3]) %in% daterange[burind])
sefdata[burind,"Value"] <- buerdat[helpind,"Value"]
sefdata[burind,"Meta"] <- "GCOS_Bueren_17891220-17970106_p"
### add Studer 
studind2 <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Studer_17970118-18011113_p"]][,c(2:4)]) )
sefdata[studind2,"Value"] <- allsefs[["GCOS_Bern_Studer_17970118-18011113_p"]]$Value
sefdata[studind2,"Meta"] <- "GCOS_Bern_Studer_17970118-18011113_p"
studind3 <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Studer_18011114-18031016_p"]][,c(2:4)]) )
sefdata[studind3,"Value"] <- allsefs[["GCOS_Bern_Studer_18011114-18031016_p"]]$Value
sefdata[studind3,"Meta"] <- "GCOS_Bern_Studer_18011114-18031016_p"
### add Sutz where Bueren + Studer is missing
temp_sutz <- read_sef(paste0(homref,"/GCOS_Sutz_17850101-18021231_ta_hom.tsv"), all=T)
sutzdat <- convert_to_stn_altitude(out=allsefs$`GCOS_Sutz_17850101-18021231_p`[,2:9], temp_dat=temp_sutz,
                                   alt=alts["GCOS_Bern_Studer_17791220-17891130_p"]-alts["GCOS_Sutz_17850101-18021231_ta"], id=NA)
sutzind <- which(daterange %in% pdate(sutzdat[,1:3]) & is.na(sefdata$Value))
helpind <- which(pdate(sutzdat[,1:3]) %in% daterange[sutzind])
sefdata[sutzind,"Value"] <- sutzdat[helpind,"Value"]
sefdata[sutzind,"Meta"] <- "GCOS_Sutz_17850101-18021231_ta"
### add Studer 
studind4 <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Studer_18031103-18270709_p"]][,c(2:4)]) )
sefdata[studind4,"Value"] <- allsefs[["GCOS_Bern_Studer_18031103-18270709_p"]]$Value
sefdata[studind4,"Meta"] <- "GCOS_Bern_Studer_18031103-18270709_p"
### add Fueter where Studer is missing
fuetind <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Fueter_18050315-18160630_p"]][,c(2:4)]) & is.na(sefdata$Value))
helpind <- which(pdate(allsefs[["GCOS_Bern_Fueter_18050315-18160630_p"]][,c(2:4)]) %in% daterange[fuetind])
sefdata[fuetind,"Value"] <- allsefs[["GCOS_Bern_Fueter_18050315-18160630_p"]]$Value[helpind]
sefdata[fuetind,"Meta"] <- "GCOS_Bern_Fueter_18050315-18160630_p"
### add Trechsel
trechsind <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Trechsel_18260401-18461231_p"]][,c(2:4)]) )
sefdata[trechsind,"Value"] <- allsefs[["GCOS_Bern_Trechsel_18260401-18461231_p"]]$Value
sefdata[trechsind,"Meta"] <- "GCOS_Bern_Trechsel_18260401-18461231_p"
trechsind2 <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Trechsel_18480101-18490930_p"]][,c(2:4)]) )
sefdata[trechsind2,"Value"] <- allsefs[["GCOS_Bern_Trechsel_18480101-18490930_p"]]$Value
sefdata[trechsind2,"Meta"] <- "GCOS_Bern_Trechsel_18480101-18490930_p"
### add Fueter where Studer/Trechsel is missing
fuetind2 <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Fueter_18190101-18331121_p"]][,c(2:4)]) & is.na(sefdata$Value))
helpind2 <- which(pdate(allsefs[["GCOS_Bern_Fueter_18190101-18331121_p"]][,c(2:4)]) %in% daterange[fuetind2])
sefdata[fuetind2,"Value"] <- allsefs[["GCOS_Bern_Fueter_18190101-18331121_p"]]$Value[helpind2]
sefdata[fuetind2,"Meta"] <- "GCOS_Bern_Fueter_18190101-18331121_p"
### add Wolf
wolfind <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Wolf_18510501-18550526_p"]][,c(2:4)]) )
sefdata[wolfind,"Value"] <- allsefs[["GCOS_Bern_Wolf_18510501-18550526_p"]]$Value
sefdata[wolfind,"Meta"] <- "GCOS_Bern_Wolf_18510501-18550526_p"
### mean difference between Trechsel and Benoit
meandiff <- dattot$`GCOS_Bern_Benoit_18371001-18530415_p` - dattot$`GCOS_Bern_Trechsel_18260401-18461231_p`
modiff <- aggregate(meandiff, list(dattot$mon),mean, na.rm=T)
# testind <- which(!is.na(meandiff))
# plot(daterange[testind],dattot$`GCOS_Bern_Benoit_18371001-18530415_p`[testind], type="l")
# points(daterange[testind],dattot$`GCOS_Bern_Trechsel_18260401-18461231_p`[testind] , type="l", col="blue")
# points(daterange[testind],dattot$`GCOS_Bern_Benoit_18371001-18530415_p`[testind] - modiff$x[month(daterange[testind])], type="l", col="green")
### add benoit where trechsel/wolf is missing, but substract a mean difference
benind <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Benoit_18371001-18530415_p"]][,c(2:4)]) & is.na(sefdata$Value))
helpind <- which(pdate(allsefs[["GCOS_Bern_Benoit_18371001-18530415_p"]][,c(2:4)]) %in% daterange[benind])
sefdata[benind,"Value"] <- allsefs[["GCOS_Bern_Benoit_18371001-18530415_p"]]$Value[helpind] - modiff$x[allsefs$`GCOS_Bern_Benoit_18371001-18530415_p`$Month[helpind]]
sefdata[benind,"Meta"] <- "GCOS_Bern_Benoit_18371001-18530415_p (with monthly substraction)"
### add Koch
kochind <- which(daterange %in% pdate(allsefs[["GCOS_Bern_Koch_18551201-18600630_p"]][,c(2:4)]))
sefdata[kochind,"Value"] <- allsefs[["GCOS_Bern_Koch_18551201-18600630_p"]]$Value
sefdata[kochind,"Meta"] <- "GCOS_Bern_Koch_18551201-18600630_p"
### add Reinhard
temp_rein <- read_sef(paste0(homref,"/GCOS_Bern_Reinhard_18601101-18631130_ta_not_homogenized.tsv"), all=T)
reindat <- convert_to_stn_altitude(out=allsefs$`GCOS_Bern_Reinhard_18601106-18631130_p`[,2:9], temp_dat=temp_rein,
                                   alt=alts["GCOS_Bern_Studer_17791220-17891130_p"]-alts["GCOS_Bern_Reinhard_18601106-18631130_p"], id=NA)
reinind <- which(daterange %in% pdate(reindat[,c(1:3)]))
sefdata[reinind,"Value"] <- reindat$Value
sefdata[reinind,"Meta"] <- "GCOS_Bern_Koch_18551201-18600630_p"
### add Burgdorf where Koch + Reinhard are missing
burgind <- which(daterange %in% pdate(allsefs$`GCOS_Burgdorf_18570101-18641231_p`[,c(2:4)]) & is.na(sefdata$Value))
helpind <- which(pdate(allsefs$`GCOS_Burgdorf_18570101-18641231_p`[,c(2:4)]) %in% daterange[burgind])
sefdata[burgind,"Value"] <- allsefs$`GCOS_Burgdorf_18570101-18641231_p`$Value[helpind]
sefdata[burgind,"Meta"] <- "GCOS_Burgdorf_18570101-18641231_p"



####  calc monthly means for break detection
momeans <- aggregate(sefdata$Value, list(sefdata$year,sefdata$mo), "mean_Xpercna")
momeans <- momeans[order(paste0(momeans$Group.1,pad2(momeans$Group.2))),]

datf <- data.frame(year=momeans$Group.1,
                   mon= momeans$Group.2,
                   day=rep(00, nrow(momeans)),
                   value= round(momeans$x,2))
write.table(datf,file=paste0("code/data_prep/daymeans/histseries/merged/hom_input_merge/","WeaR_BERmerged_p",".dat"),quote=F,row.names = F,col.names = F)
## comp series
ekf_p <- read.table("code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_p.dat")
dind <- paste0(datf$year,pad2(datf$mon)) %in% paste0(ekf_p$V1,pad2(ekf_p$V2))
eind <- paste0(ekf_p$V1,pad2(ekf_p$V2)) %in% paste0(datf$year,pad2(datf$mon))
diff <- data.frame(date=paste0(datf$year,pad2(datf$mon))[dind],diff=(datf$value[dind]- ekf_p$V4[eind]),ekf=ekf_p$V4[eind],hist=datf$value[dind])

png(paste0(plotpath,"Diff_EKF400_ber_nothom_p_",Sys.Date(),".png"), width=2500)
plot(1:nrow(diff),diff$diff, type="l", xaxt="n")
title("Difference EKF400 - Merged Bern P (not hom)")
abline(v=seq(1,150*12,5*12), lty=3, col="gray65")
axis(side=1, at=seq(1, nrow(diff),12),momeans$Group.1[seq(1,nrow(diff),12)])
dev.off()

#########################################################################################################
## check for change points with ref sereis
#########################################################################################################

if (!"WeaR_BERmerged_p" %in% list.dirs("code/data_prep/daymeans/histseries/merged/hom_output_merge/",full.names=FALSE)) {
  dir.create(path = "code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_BERmerged_p/")}
FindU.wRef(Bseries="code/data_prep/daymeans/histseries/merged/hom_input_merge/WeaR_BERmerged_p.dat",MissingValueCode = "NA", p.lev=0.95,Iadj=10000,Mq=10,Ny4a=0,
           output ="code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_BERmerged_p/WeaR_BERmerged_p",
           Rseries="code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_p.dat")

#########################################################################################################
## reestimate significant change points
#########################################################################################################
StepSize.wRef(Bseries="code/data_prep/daymeans/histseries/merged/hom_input_merge/WeaR_BERmerged_p.dat",
              Rseries="code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_p.dat",
              InCs = "code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_BERmerged_p/WeaR_BERmerged_p_mCs.txt",
              MissingValueCode = "NA", p.lev=0.95,Iadj=10000,Mq=10,Ny4a=0,
              output = "code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_BERmerged_p/WeaR_BERmerged_p")

#########################################################################################################
## create final homogenized sef-files with adjustements estimated above 
#########################################################################################################

# read hom vals
Fdat <- read.table("code/data_prep/daymeans/histseries/merged/hom_output_merge/WeaR_BERmerged_p/WeaR_BERmerged_p_F.dat")
qmt <- 5 #ifelse(all(is.na(Fdat[,11])),5,11) -> don't use the qm BERed estimates, because they are estimated based on the EKF reference
## set Fdat values to NA, if they are followed by a large number of NAs...

homadj_monthly <- Fdat[,qmt] - Fdat[,3] 
# expand monthly values to daily vals
mimo <- Fdat[,2]+15
mohelp <- as.Date(paste0(substr(mimo,1,4),"-",substr(mimo,5,6),"-",substr(mimo,7,8)))
lmo <- lubridate::days_in_month(as.Date(paste0(substr(Fdat[nrow(Fdat),2],1,4),"-",substr(Fdat[nrow(Fdat),2],5,6),"-01")))
dhelp <- seq(as.Date(paste0(substr(Fdat[1,2],1,4),"-",substr(Fdat[1,2],5,6),"-01")),
             as.Date(paste0(substr(Fdat[nrow(Fdat),2],1,4),"-",substr(Fdat[nrow(Fdat),2],5,6),"-",lmo)),
             by="day")
xout <- which(dhelp %in% pdate(sefdata[,1:3]))

## don't use spline. because it makes weird step to next period...
homadj_daily <- homadj_monthly[match(substr(dhelp[xout],1,7),substr(mohelp,1,7))]
# add hom adjust
sefdata[,"Value"] <- round(sefdata[,"Value"] + homadj_daily,2)
pipe <- ifelse(sefdata[,"Meta"]=="","","|")
sefdata[,"Meta"] <- paste0(sefdata[,"Meta"],pipe,"homadj=",round(homadj_daily,2))

## plot 
momeans_test <- aggregate(sefdata$Value, list(sefdata$year,sefdata$mo), "mean_Xpercna", na=0.25)
momeans_test <- momeans_test[order(paste0(momeans_test$Group.1,pad2(momeans_test$Group.2))),]
datehelp <- as.Date(paste0(momeans_test$Group.1,"-",pad2(momeans_test$Group.2),"-","15"))
ekftest <- read.table("code/data_prep/daymeans/histseries/merged/hom_input_merge/EKF400_REF_p.dat")
ekfind <- which(paste0(ekftest$V1,"-",pad2(ekftest$V2)) %in% paste0(momeans_test$Group.1,"-",pad2(momeans_test$Group.2)))

png(paste0(plotpath,"Diff_EKF400_hombern_p_",Sys.Date(),".png"), width=2500)
plot(datehelp,momeans_test$x - ekftest$V4[ekfind], type="l", ylab="hPa")
title("Difference EKF400 - Merged Bern P")
abline(v=datehelp[seq(1,12*140,12*5)], lty=3, col="gray65")
dev.off()


### plot daily sefdata
#### temperature
stncols <- c("black","blue3","cornflowerblue","turquoise1","forestgreen","green","yellowgreen","yellow","orange",
             "darkred","tomato","pink", "purple","magenta")
ser <- sefdata
doys <- lubridate::yday(pdate(ser[,1:3]))
doymeans <- aggregate(ser$Value, list(doys),"mean", na.rm=T)
dayanoms <- ser$Value-doymeans$x[doys]
metahelp <- gsub("|homadj=.*|monthly_mean_too_high|","",ser$Meta)
metahelp <- gsub("\\|","", metahelp)
mu <- unique(metahelp)
mu <- mu[!grepl("NANA",mu)]
colsel <- stncols[match(metahelp,mu)]

png(paste0(plotpath,"histperiod_p_bern_final_",Sys.Date(),".png"), width=3000,res=180)
par(mar=c(3,2,2,10))
plot(1:nrow(ser),dayanoms, pch=20, col=colsel,xlab="", ylab="", xaxt="n",cex=0.1, ylim=c(-30,30))
axis(side=1, at=which(yday(daterange)==1)[seq(1,length(which(yday(daterange)==1)),5)],
     labels = substr(as.character(daterange[which(yday(daterange)==1)[seq(1,length(which(yday(daterange)==1)),5)]]),1,4))
mtext(side=3,line=-1.2,text="Bern P", cex=0.8)
abline(v=which(yday(daterange)==1)[seq(1,length(which(yday(daterange)==1)),5)], lty=3, col="gray55")
par(xpd=TRUE)
legend("right", inset=c(-0.08,0), legend=mu, col=stncols,pch=20, bty="n", cex=0.8)
par(xpd=F)
dev.off()

write_sef(Data = sefdata[, 1:7],
          outpath = finpath,
          variable = "p",
          cod = "BER",
          nam = "Bern",
          lat = lats[1],
          lon = lons[1],
          alt = alts[1],
          link = "",
          sou = "WeaR",
          units = "hPa",
          stat = "mean",
          metaHead = "homogenised merged serie from CHIMES",
          meta=sefdata$Meta,
          period = "day",
          time_offset =0,
          keep_na = TRUE,
          note="hom")


