#!/bin/bash

set -o errexit
set -o pipefail

cd /you/yourworkingdirectory

date=2022-09-06
start=1961-01-01
end=2020-12-31
scen=scen3
scen2=scen3
expdate=ARM_run_${date}
scenario=${scen}
dist=Gower
stnval=all
calonly=T
climoff=T
# localization TRUE/FALSE is set in 053_EnKF.R script
L=200
Z=1000
R=obsest

echo "Calculate analogs"
# the masterscript sources the scripts 011,012,013,020,030 for the analogs calculation.
Rscript C010_masterscript.R $scen $stnval $dist $start $end $calonly $climoff

echo "Create netcdf"
# resamples the fields to get an analog reconstruction
. C040_extract_netcdf_loop.sh ${date} ${start} ${end} ${scen} ${dist}
 
echo "Run Kalman Filter"
# assimilate temperature measurements
. C050_EnKF_loop.sh ${scen} ${date} ${start} ${end} ${L} ${Z} ${climoff} ${dist}
    
echo "Run Qmapping" 
# this runs scenario wise - not started based on all_hist, but for individual networks. Note that arguments are set at the beginning of the script.
. C060_Qmap_fit.R 
        
echo "merge temperature files for validation"
# define infile_folder!
ncrcat infile_folder/${date}_temp_EnKF_deseas* infile_folder/${date}_temp_EnKF_deseas_${start}-${end}.nc

echo "Start Validation"
# script is called from the console. Calls a dask clients that needs to be set up according to cores and RAM.
# for line based run uncomment sys.args -line and add path to validation file when running the script
python3 C071_validation_precip.py #infile_folder/${date}_Qmap_${start}-${end}.nc
python3 C072_validation_temp.py #infile_folder/${date}_temp_EnKF_deseas_${start}-${end}.nc
 




