#!/anaconda3/envs/my_env/bin/python

# create yearly netCDF from best analogue dates
# 2022-03-11
# -----------------------------------------------

# import packages
import numpy as np
import pandas as pd
import xarray as xr
import time
import sys
import glob
import netCDF4
import datetime
import os

# for line-based run: define script input manually (comment lines for console run)
#sys.argv = ["", "/scratch3/noemi/wear/swiss_arm/code/output/ARM_run_2022-04-07/scen6/",1961,"1961-01-01","RMSE"]

print(sys.argv)
# get start time
stm = time.time()

# check if input arguments exist
if len(sys.argv) < 2:
    sys.exit("indicate folder path to analogue dates")

start = str(sys.argv[3])
dist = str(sys.argv[4])

# read folder
dates_ifolder = str(sys.argv[1])  # get first argument passed to script (folder of analogue dates)
print(dates_ifolder)

    
# create folder for fields
outdir = dates_ifolder + "/ARM_precip_"+  time.strftime("%Y-%m-%d") + "/"
print(outdir)
if not os.path.isdir(outdir):
    try:
        os.mkdir(outdir)
    except OSError:
        print("")
    else:
        print("")


#--------------------------------------------------------------------------------------------------------------------
# READ DATA
# --------------------------------------------------------------------------------------------------------------------
print("read readme")
print(start)
print(dist)
print(glob.glob(f"{dates_ifolder}*readme*{start}*{dist}*.txt"))

analog_readme = np.array(pd.read_csv(glob.glob(f"{dates_ifolder}*readme*{start}*{dist}*.txt")[0], sep=" "))  # read csv with "readme" in filename
offset = analog_readme[np.argwhere(analog_readme[:,0]=='offset:').item(0),1]
endCal = pd.to_datetime(analog_readme[np.argwhere(analog_readme[:,0]=='endCal:').item(0),1])

print(glob.glob(f'{dates_ifolder}*analog.dates*{start}*{dist}*.txt'))
analog_dates = pd.read_csv(glob.glob(f"{dates_ifolder}*analog.dates*{start}*{dist}*.txt")[0], sep=" ")  # read csv with "analog.dates" in filename

# get target dates and best analogue dates
dates = pd.to_datetime(analog_dates["date"].values)
analog_1_dates = pd.to_datetime(analog_dates["1"], unit='D', origin=pd.Timestamp("1970-01-01"))  # get vector of best analogue dates
analog_1_dates = analog_1_dates.dt.strftime("%Y-%m-%d")

print(endCal)
print(dates)
    
year= int(sys.argv[2])
selyear = dates.year == year
    
############################################
# read raster data
###########################################
obs_ifolder = "/scratch3/noemi/wear/swiss_arm/code/data_input/"

print("read data")
precip_ifile =  "RhiresD_ch01h.swiss.lv95_1961-01-01-2020-12-31_masked.nc"
precip_obs = xr.open_dataset(obs_ifolder + precip_ifile)
precip_obs = precip_obs["RhiresD"]

## change time format
precip_obs["time"] = precip_obs.time.dt.strftime("%Y-%m-%d")

print("create preciperature grids from best analogue dates")
precip_analogs = precip_obs.reindex(time=analog_1_dates[selyear])    
precip_analogs['time']  = pd.to_datetime(analog_dates["date"][selyear]).values
precip_analogs = precip_analogs.to_dataset()
precip_analogs = precip_analogs.rename_vars(name_dict=dict(RhiresD="precip"))

# write raster data to netCDF files
print("write data to netCDF files")
dist = np.argwhere(analog_readme[:,0]=='distance:').item(0)
scal = np.argwhere(analog_readme[:,0]=='startCal:').item(0)
scen = np.argwhere(analog_readme[:,0]=='scenario:').item(0)

print(scen)
precip_analogs.to_netcdf(outdir + analog_readme[0, 1] + "_precip_analogs_" + str(sys.argv[2]) + "-01-01-" + str(sys.argv[2])  + "-12-31_" + analog_readme[dist, 1] + "_" + analog_readme[scen, 1]  + ".nc", format='NETCDF4')

del precip_analogs, precip_obs
etm = time.time()
print("precip fields created in: " + str(round((etm-stm)//60)) + " minutes, " + str(round((etm-stm) % 60, 2)) + " seconds")
print("files are in: " + outdir)

### END
