### 060 Qmap ###
### quantile mapping (for precipitation) 

rm(list=ls())

setwd("/scratch3/noemi/wear/swiss_arm/code/")

scen <- "scen1781"
yrlist <- list(scen1 =  c(1767,1795), scen2 = c(1796,1812), scen3 = c(1813,1833), scen1781 = c(1763,1789), scen1834 = c(1834,1855), scen1856 = c(1856,1863))
yrs <- yrlist[[scen]]
date <- "2023-02-17"
date2 <- "2023-02-17"

args <- c(paste0("output/ARM_run_",date,"/",scen,"/ARM_precip_",date,"/",date,"_precip_analogs_1961-01-01-2020-12-31_Gower_",scen,".nc"),
          paste0("output/ARM_run_",date2,"/all_hist/ARM_precip_",date2,"/",date2,"_precip_analogs_",yrs[1],"-01-01-",yrs[2],"-12-31_Gower_all_hist.nc"), 
          yrs[1], yrs[2])


library(raster);library(qmap)
library(ncdf4);library(lubridate)
library(foreach);library(iterators);library(parallel);library(doParallel)

source("/scratch3/noemi/wear/swiss_arm/code/013_helpfun.R")

print(args[1])

qmapset <- list(
  input_raster_obs = "data_input/RhiresD_ch01h.swiss.lv95_1961-01-01-2020-12-31_masked.nc",
  input_raster_rec = args[1],
  input_raster_hist = args[2],
  start = args[3],
  end = args[4],
  method = "QUANT",
  qstep = 0.01,
  wet.day=TRUE
)

## create output folder for validation
seldate <- Sys.Date()
subDir <- paste("Qmap",seldate,sep="_")
dirpath <- gsub("ARM_precip.*","",dirname(qmapset$input_raster_rec))
if(!dir.exists(paste0(dirpath,subDir))){dir.create(file.path(dirpath,subDir), showWarnings = F)}
outfolder <- paste0(dirpath,subDir,"/")

# get time
ptm <- proc.time()

#############################################################################################################
### 1. READ DATA
#############################################################################################################
print("read data")

sdate = as.Date("1961-01-01")
edate <- "2020-12-31"

inraster_obs <- qmapset$input_raster_obs
inraster_rec <- qmapset$input_raster_rec
inraster_hist <- qmapset$input_raster_hist

varname_rec <- "precip"
varname_obs <- "RhiresD"

nc_obs <- nc_open(inraster_obs)
nc_rec <- nc_open(inraster_rec)
nc_hist <- nc_open(inraster_hist)

#############################################################################################################
##### define new files ##########
#############################################################################################################

##  create two files
filename_hist <- paste(outfolder,"/",seldate,"_precip_QMAP_",qmapset$start,"-01-01-",qmapset$end,"-12-31.nc",sep="")
filename_ref <- paste(outfolder,"/",seldate,"_precip_QMAP_",sdate,"-",edate,".nc",sep="")

N <- ncvar_get(nc_obs, "N")
E <- ncvar_get(nc_obs, "E")
time_obs <- ncvar_get(nc_obs,"time")
dates_obs <- as.Date(time_obs, origin = "1900-01-01")

time_hist <- ncvar_get(nc_hist,"time")
dates_hist <- as.Date(time_hist, origin = "1763-01-01")

nx <- length(E)
ny <- length(N)
lon1 <- ncdim_def("E", "meter", E)
lat2 <- ncdim_def("N", "meter", N)

time_ref <- ncdim_def("time","days since 1900-01-01", time_obs, unlim=TRUE)
mv <- -999 #missing value to use
var_temp_ref <- ncvar_def("precip", "mm/day", list(lon1, lat2, time_ref), longname="daily precip sums post-processed", mv) 
ncnew_ref <- nc_create(filename_ref, list(var_temp_ref))

time_hist2 <- time_hist[year(dates_hist) >= qmapset$start & year(dates_hist) <= qmapset$end]
time_hist_def <- ncdim_def("time","days since 1763-01-01", time_hist2, unlim=TRUE)
mv <- -999 #missing value to use
var_temp_hist <- ncvar_def("precip", "mm/day", list(lon1, lat2, time_hist_def), longname="daily precip sums post-processed", mv) 
ncnew_hist <- nc_create(filename_hist, list(var_temp_hist))

#############################################################################################################
### 2. DO QUANTILE MAPPING
#############################################################################################################
print("do quantile mapping")

get_15day_factor <- function(dates){
  samp <- c(rep(1:24, each = 15),1,5,10,15,20) ## create samples of 15 or 16 days
  fifteendays <- samp[order(samp)] ## order again
  doys <- as.numeric(get_date_factors_obj(dates)$jdays)
  fifteenday_doys <- fifteendays[doys]
  return(fifteenday_doys)
}

# define qmap functions
qmapping <- function(obs_vp, rec_raw, rec_hist, dates_ref, dates_hist){
  
  fittedvals_rec = rep(NA, length(rec_raw))
  fittedvals_hist = rep(NA, length(rec_hist))
  
  if(any(is.na(rec_raw))){
    return(list(ref = fittedvals_rec, hist = fittedvals_hist))
  }
  else{
    for(wk in 1:24){ ## this is 365 divided by 5
      
      doy_ref <- get_15day_factor(dates_ref)
      doy_hist <- get_15day_factor(dates_hist)
      
      wk_ext <- (wk-1):(wk+1)
      wk_ext[wk_ext < 1] <- wk_ext[wk_ext < 1] + 24
      wk_ext[wk_ext > 24] <- wk_ext[wk_ext  > 24] - 24
      
      sind_ref <- which(doy_ref %in% wk_ext)
      alldoys_ref <- which(doy_ref == wk)
      alldoys_hist <- which(doy_hist == wk)
      
      fitmod <- fitQmap(obs = obs_vp[sind_ref],
                        mod = rec_raw[sind_ref],
                        method = qmapset$method,
                        qstep = qmapset$qstep,
                        wet.day = qmapset$wet.day)
      
      fittedvals_rec[alldoys_ref] <- round(doQmap(rec_raw[alldoys_ref],fitmod),2)
      
      if(any(is.na(rec_hist[alldoys_hist]))){
        alldoys_hist <- alldoys_hist[!is.na(rec_hist[alldoys_hist])]
      } 
      
      fittedvals_hist[alldoys_hist] <- round(doQmap(rec_hist[alldoys_hist],fitmod),2)
    }
    return(list(ref = fittedvals_rec, hist = fittedvals_hist))  
  }
}

chunks <- seq(1,length(E),3)
print(chunks)

##### loop over several longitudes ########

registerDoParallel(cores=floor(detectCores()*0.2)) 
ii <- 100

sellon <- 3
startind <- 1 
endlen <- length(startind:which(dates_hist == paste0(qmapset$end,"-12-31")))

for(ii in chunks){
  print(paste0("calc chunk: ", ii))
  cklen <- ifelse(ii == 370,1,sellon)
  tictoc::tic()
  vals_obs <- array(ncvar_get(nc_obs, varid = varname_obs, start = c(ii,1,1), count = c(cklen,-1,-1)), dim = c(cklen,ny,length(dates_obs)))
  vals_rec <- array(ncvar_get(nc_rec, varid = varname_rec, start = c(ii,1,1), count = c(cklen,-1,-1)), dim = c(cklen,ny,length(dates_obs)))
  vals_hist <- array(ncvar_get(nc_hist, varid = varname_rec, 
                         start = c(ii,1,startind), count = c(cklen,-1,-1)), dim = c(cklen,ny,length(dates_hist)))
  tictoc::toc()
  
  if(!all(is.na(vals_rec))){
    
    obs_list <- split(vals_obs,  seq(nrow(vals_obs)*ncol(vals_obs)))
    rec_list <- split(vals_rec,  seq(nrow(vals_rec)*ncol(vals_rec)))
    hist_list <- split(vals_hist,  seq(nrow(vals_hist)*ncol(vals_rec)))
    
    print("do qmap on chunk")
    tictoc::tic()
    qmap_list <- foreach(a = 1:length(obs_list)) %dopar% {
      qmapping(obs_vp = obs_list[[a]],rec_raw = rec_list[[a]], rec_hist = hist_list[[a]], dates_ref = dates_obs,
               dates_hist = dates_hist[1:endlen])
    }
    tictoc::toc()
    
    qmap_ref <- array(t(sapply(qmap_list, function(x) x$ref)),dim = c(nrow(vals_obs),ncol(vals_obs),length(dates_obs)))
    qmap_hist <- array(t(sapply(qmap_list, function(x) x$hist)),dim = c(nrow(vals_hist),ncol(vals_hist),length(dates_hist)))
    
    print("write values")
    ncvar_put(ncnew_ref, var_temp_ref, qmap_ref,  start = c(ii,1,1), count = c(cklen,-1,-1))
    ncvar_put(ncnew_hist, var_temp_hist, qmap_hist, start = c(ii,1,1), count = c(cklen,-1,-1))
    
  }
}

## write a file for evaluating the training
nc_close(ncnew_ref)
nc_close(ncnew_hist)

etm = proc.time()

paste("calculation time:",(etm[3]-ptm[3])%/%60,"minutes", round((etm[3]-ptm[3])%%60),"seconds" ,sep=" ")
print("QMAP fit done")
